#ifndef YCF_YIELDING_C_FUN_HELPERS
#define YCF_YIELDING_C_FUN_HELPERS 1
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

/*
 * YCF_GCC_DIAG_ON and YCF_GCC_DIAG_OFF can be used to temporarly
 * disable a gcc or clang warning in a file.
 *
 * Example:
 * YCF_GCC_DIAG_OFF(unused-function)
 * static int test(){ return 0;}
 * YCF_GCC_DIAG_ON(unused-function)
 *
 * These macros were orginally authored by Jonathan Wakely and has
 * been modified by Patrick Horgan.
 *
 * Source: http://dbp-consulting.com/tutorials/SuppressingGCCWarnings.html
 *
 */
#if defined(_MSC_VER)
#  define YCF_GCC_DIAG_OFF(x) __pragma(warning(push, 0))
#  define YCF_GCC_DIAG_ON(x)  __pragma(warning(pop))
#elif ((__GNUC__ * 100) + __GNUC_MINOR__) >= 402
#define YCF_GCC_DIAG_STR(s) #s
#define YCF_GCC_DIAG_JOINSTR(x,y) YCF_GCC_DIAG_STR(x ## y)
# define YCF_GCC_DIAG_DO_PRAGMA(x) _Pragma (#x)
# define YCF_GCC_DIAG_PRAGMA(x) YCF_GCC_DIAG_DO_PRAGMA(GCC diagnostic x)
# if ((__GNUC__ * 100) + __GNUC_MINOR__) >= 406
#  define YCF_GCC_DIAG_OFF(x) YCF_GCC_DIAG_PRAGMA(push) \
      YCF_GCC_DIAG_PRAGMA(ignored YCF_GCC_DIAG_JOINSTR(-W,x))
#  define YCF_GCC_DIAG_ON(x) YCF_GCC_DIAG_PRAGMA(pop)
# else
#  define YCF_GCC_DIAG_OFF(x) YCF_GCC_DIAG_PRAGMA(ignored YCF_GCC_DIAG_JOINSTR(-W,x))
#  define YCF_GCC_DIAG_ON(x)  YCF_GCC_DIAG_PRAGMA(warning YCF_GCC_DIAG_JOINSTR(-W,x))
# endif
#else
# define YCF_GCC_DIAG_OFF(x)
# define YCF_GCC_DIAG_ON(x)
#endif
#ifdef __GNUC__
#  if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 5) || defined(__clang__)
#    define YCF_GCC_ATTRIBUTE_UNUSED __attribute__ ((unused))
#  else
#    define YCF_GCC_ATTRIBUTE_UNUSED
#  endif
#else
#  define YCF_GCC_ATTRIBUTE_UNUSED
#endif

typedef void* (*ycf_yield_alloc_type) (size_t,void*);
typedef void (*ycf_yield_free_type) (void*,void*);

struct ycf_alloc_data {
  size_t size;
  size_t max_size;
  int needs_freeing;
  char* data;
};

#define YCF_ALLOC_NEXT_BLOCK() (\
 ycf_frame_alloc_data.data == NULL \
   ? NULL \
   : ((void*)(&ycf_frame_alloc_data.data[ycf_frame_alloc_data.size]))\
)
#define YCF_ALLOC_NEXT_MAX_SIZE() (\
 ycf_frame_alloc_data.data == NULL \
   ? ycf_frame_alloc_data.max_size \
   : (ycf_frame_alloc_data.max_size - ycf_frame_alloc_data.size)\
)

/* Macros for special code sections */
#define ON_SAVE_YIELD_STATE ON_SAVE_YIELD_STATE
#define ON_RESTORE_YIELD_STATE ON_RESTORE_YIELD_STATE
#define ON_DESTROY_STATE ON_DESTROY_STATE
#define ON_RETURN ON_RETURN
#define ON_DESTROY_STATE_OR_RETURN ON_DESTROY_STATE_OR_RETURN
#define YCF_SPECIAL_CODE_START(PARAM) \
   /*special_code_start:PARAM*/       \
   if(0){
#define YCF_SPECIAL_CODE_END() \
   }\
   /*special_code_end*/

YCF_GCC_ATTRIBUTE_UNUSED
static void* ycf_stack_alloc(size_t size,
                             struct ycf_alloc_data* data,
                             ycf_yield_alloc_type allocator,
                             void* ycf_yield_alloc_free_context,
                             size_t default_stack_size){
  void * ret = NULL;
  if (data->data == NULL) {
    if (default_stack_size == 0) {
      fprintf(stderr, "ycf_alloc: not enough stack!! (max size = 0)\n");
      exit(1);
    }
    data->data = allocator(default_stack_size, ycf_yield_alloc_free_context);
    data->needs_freeing = 1;
    data->max_size = default_stack_size;
    data->size = 0;
  }
  if (data->size + size > data->max_size) {
    fprintf(stderr, "ycf_alloc: not enough stack! (max size = %zu)\n", default_stack_size);
    exit(1);
  }
  ret = &data->data[data->size];
  data->size = data->size + size + (sizeof(void * ) - (size % sizeof(void * ))) % sizeof(void * );
  return ret;
}
YCF_GCC_ATTRIBUTE_UNUSED
static void ycf_destroy_stack_allocator(struct ycf_alloc_data* data,
                                        ycf_yield_free_type freer,
                                        void* ycf_yield_alloc_free_context){
  if(data->needs_freeing){
    freer(data->data, ycf_yield_alloc_free_context);
  }
}


#include <limits.h>
#define YCF_MAX_NR_OF_REDS LONG_MAX
#define YCF_NR_OF_REDS_LEFT() ycf_nr_of_reductions
#define YCF_SET_NR_OF_REDS_LEFT(NEW_NR_OF_REDS_LEFT) \
  do{ycf_nr_of_reductions = (NEW_NR_OF_REDS_LEFT);}while(0)

#define YCF_GET_EXTRA_CONTEXT() ycf_extra_context

#define YCF_IS_YIELDED(CTX) (CTX != NULL)

#define YCF_YIELD_CODE_GENERATED 1


/* end of YCF_YIELDING_C_FUN_HELPERS guard */
#endif



struct gen_ycf_trap_state_for_ets_insert_2_list_lock_tbl;

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  void  ets_insert_2_list_lock_tbl_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_2_list_lock_tbl_ycf_gen_destroy(void* ycf_my_trap_state_param);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */



static void ets_insert_2_list_lock_tbl_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,Eterm table_id_N127_N135,
                                       Process* p_N128_N136,
                                       Uint bif_ix_N129_N137,
                                       ets_insert_2_list_status on_success_status_N130_N138);



struct gen_ycf_trap_state_for_ets_insert_2_list;

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  BIF_RETTYPE  ets_insert_2_list_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_2_list_ycf_gen_destroy(void* ycf_my_trap_state_param);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

static BIF_RETTYPE ets_insert_2_list_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,Process* p_N88_N118,
                                     Eterm table_id_N89_N119,
                                     DbTable *tb_N90_N120,
                                     Eterm list_N91_N121,
                                     int is_insert_new_N92_N122);



struct gen_ycf_trap_state_for_ets_insert_2_list_insert_db_term_list;

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  void  ets_insert_2_list_insert_db_term_list_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_2_list_insert_db_term_list_ycf_gen_destroy(void* ycf_my_trap_state_param);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */



static void ets_insert_2_list_insert_db_term_list_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,DbTable* tb_N70_N75,
                                                  void* list_N71_N76);



struct gen_ycf_trap_state_for_ets_insert_new_2_dbterm_list_has_member;

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  int  ets_insert_new_2_dbterm_list_has_member_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_new_2_dbterm_list_has_member_ycf_gen_destroy(void* ycf_my_trap_state_param);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */



static int ets_insert_new_2_dbterm_list_has_member_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,DbTable* tb_N58_N65, void* db_term_list_N59_N66);



struct gen_ycf_trap_state_for_ets_insert_2_list_copy_term_list;

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

    static  void  *  ets_insert_2_list_copy_term_list_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_2_list_copy_term_list_ycf_gen_destroy(void* ycf_my_trap_state_param);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

static void* ets_insert_2_list_copy_term_list_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,DbTableMethod* meth_N42_N49,
                                              int compress_N43_N50,
                                              int keypos_N44_N51,
                                              Eterm list_N45_N52);



struct gen_ycf_trap_state_for_ets_insert_2_list_destroy_copied_dbterms;

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  void  ets_insert_2_list_destroy_copied_dbterms_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_2_list_destroy_copied_dbterms_ycf_gen_destroy(void* ycf_my_trap_state_param);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */


/* This function is called both as is, and as YCF transformed. */
static void ets_insert_2_list_destroy_copied_dbterms_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,DbTableMethod* meth_N31_N36,
                                                     int compressed_N32_N37,
                                                     void* db_term_list_N33_N38);



struct gen_ycf_trap_state_for_ets_insert_2_list_from_p_heap;

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  int  ets_insert_2_list_from_p_heap_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_2_list_from_p_heap_ycf_gen_destroy(void* ycf_my_trap_state_param);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */



static int ets_insert_2_list_from_p_heap_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,DbTable* tb_N22_N27, Eterm list_N23_N28);



struct gen_ycf_trap_state_for_ets_insert_new_2_list_has_member;

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  int  ets_insert_new_2_list_has_member_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_new_2_list_has_member_ycf_gen_destroy(void* ycf_my_trap_state_param);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */



static int ets_insert_new_2_list_has_member_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,DbTable* tb_N12_N17, Eterm list_N13_N18);



struct gen_ycf_trap_state_for_ets_insert_2_list_check;

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  long  ets_insert_2_list_check_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_2_list_check_ycf_gen_destroy(void* ycf_my_trap_state_param);
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

static long ets_insert_2_list_check_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,int keypos_N3_N7, Eterm list_N4_N8);



struct gen_ycf_trap_state_for_ets_insert_2_list_check {
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    Eterm lst_N1_N5;
    long i_N2_N6;int keypos_N3_N7; Eterm list_N4_N8;
};


/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_2_list_check_ycf_gen_destroy(void* ycf_my_trap_state_param){
     {
     struct gen_ycf_trap_state_for_ets_insert_2_list_check* ycf_my_trap_state = ycf_my_trap_state_param;
     
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    Eterm lst_N1_N5;
    long i_N2_N6;int keypos_N3_N7; Eterm list_N4_N8;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    lst_N1_N5 = ycf_my_trap_state->lst_N1_N5;
    i_N2_N6 = ycf_my_trap_state->i_N2_N6;
    keypos_N3_N7 = ycf_my_trap_state->keypos_N3_N7;
    list_N4_N8 = ycf_my_trap_state->list_N4_N8;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_destroy_stack_allocator(&ycf_frame_alloc_data, ycf_yield_free, ycf_yield_alloc_free_context);
     ycf_yield_free(ycf_my_trap_state, ycf_yield_alloc_free_context);
     }
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  long  ets_insert_2_list_check_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context){
     struct gen_ycf_trap_state_for_ets_insert_2_list_check* ycf_my_trap_state = *ycf_trap_state;
return ets_insert_2_list_check_ycf_gen_yielding(ycf_number_of_reduction_param,
                    ycf_trap_state,
                    ycf_extra_context,
                    ycf_my_trap_state->ycf_yield_alloc,
                    ycf_my_trap_state->ycf_yield_free,
                    ycf_my_trap_state->ycf_yield_alloc_free_context,
                    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size,
                    ycf_my_trap_state->ycf_stack_alloc_data
                    ,ycf_my_trap_state->keypos_N3_N7,ycf_my_trap_state->list_N4_N8)
;
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

#define YCF_IN_YIELDING_FUN 1
#undef YCF_STACK_ALLOC
#define YCF_STACK_ALLOC(SIZE) \
    ycf_stack_alloc(SIZE,\
                    &ycf_frame_alloc_data,\
                    ycf_yield_alloc, ycf_yield_alloc_free_context,\
                     ycf_stack_alloc_size_or_max_size)

/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(uninitialized)
/* clang-format on */
static long ets_insert_2_list_check_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,int keypos_N3_N7, Eterm list_N4_N8)
{
    Eterm lst_N1_N5;
    long i_N2_N6;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;

{
  
  ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
  ycf_frame_alloc_data.size = 0;
  ycf_frame_alloc_data.max_size = ycf_stack_alloc_size_or_max_size;
  ycf_frame_alloc_data.data = ycf_stack_alloc_data;
  ycf_frame_alloc_data.needs_freeing = 0;
  if(*ycf_trap_state != NULL){
     struct gen_ycf_trap_state_for_ets_insert_2_list_check* ycf_my_trap_state = *ycf_trap_state;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    lst_N1_N5 = ycf_my_trap_state->lst_N1_N5;
    i_N2_N6 = ycf_my_trap_state->i_N2_N6;
    keypos_N3_N7 = ycf_my_trap_state->keypos_N3_N7;
    list_N4_N8 = ycf_my_trap_state->list_N4_N8;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
     switch(ycf_trap_location){
case 1: goto ycf_yield_location_label_1;

}
  }
}
{

}
    lst_N1_N5 = THE_NON_VALUE;
    i_N2_N6 = 0;
    for (lst_N1_N5 = list_N4_N8; is_list(lst_N1_N5); lst_N1_N5 = CDR(list_val(lst_N1_N5)))
{
{
ycf_nr_of_reductions = ycf_nr_of_reductions - (1);
if (ycf_nr_of_reductions <= 0) {
  ycf_trap_location = 1;
  goto ycf_do_yield_label_ets_insert_2_list_check;
  ycf_yield_location_label_1:;
}

}
    {
        i_N2_N6++;
        if (is_not_tuple(CAR(list_val(lst_N1_N5))) ||
            (arityval(*tuple_val(CAR(list_val(lst_N1_N5)))) < keypos_N3_N7))
{
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

            return -1;
}

}
    }

}
    if (lst_N1_N5 != NIL)
{
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

        return -1;
}

}
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

    return i_N2_N6;
}

{
  struct gen_ycf_trap_state_for_ets_insert_2_list_check* ycf_my_trap_state;
  ycf_do_yield_label_ets_insert_2_list_check:;
  
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */
  if (*ycf_trap_state == NULL) {
    ycf_my_trap_state = ycf_yield_alloc(sizeof(struct gen_ycf_trap_state_for_ets_insert_2_list_check), ycf_yield_alloc_free_context);
      } else {
    ycf_my_trap_state = *ycf_trap_state;
  }
      ycf_my_trap_state->ycf_yield_alloc = ycf_yield_alloc;
    ycf_my_trap_state->ycf_yield_free = ycf_yield_free;
    ycf_my_trap_state->ycf_yield_alloc_free_context = ycf_yield_alloc_free_context;
    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size = ycf_stack_alloc_size_or_max_size;
    ycf_my_trap_state->ycf_stack_alloc_data = ycf_stack_alloc_data;
    ycf_my_trap_state->ycf_trap_location = ycf_trap_location;
    ycf_my_trap_state->ycf_nr_of_reductions = ycf_nr_of_reductions;
    ycf_my_trap_state->ycf_frame_alloc_data = ycf_frame_alloc_data;
    ycf_my_trap_state->lst_N1_N5 = lst_N1_N5;
    ycf_my_trap_state->i_N2_N6 = i_N2_N6;
    ycf_my_trap_state->keypos_N3_N7 = keypos_N3_N7;
    ycf_my_trap_state->list_N4_N8 = list_N4_N8;

  *ycf_nr_of_reductions_param = ycf_nr_of_reductions;
  *ycf_trap_state = ycf_my_trap_state;
      {
    static   long  const ycf_unused_ret_value;
    return ycf_unused_ret_value;
  }

}
}
/* clang-format off */
YCF_GCC_DIAG_ON(uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */

#undef YCF_STACK_ALLOC
#undef YCF_IN_YIELDING_FUN



struct gen_ycf_trap_state_for_ets_insert_new_2_list_has_member {
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    Eterm lst_N9_N14;
    Eterm lookup_ret_N10_N15;
    DbTableMethod* meth_N11_N16;DbTable* tb_N12_N17; Eterm list_N13_N18;
};


/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_new_2_list_has_member_ycf_gen_destroy(void* ycf_my_trap_state_param){
     {
     struct gen_ycf_trap_state_for_ets_insert_new_2_list_has_member* ycf_my_trap_state = ycf_my_trap_state_param;
     
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    Eterm lst_N9_N14;
    Eterm lookup_ret_N10_N15;
    DbTableMethod* meth_N11_N16;DbTable* tb_N12_N17; Eterm list_N13_N18;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    lst_N9_N14 = ycf_my_trap_state->lst_N9_N14;
    lookup_ret_N10_N15 = ycf_my_trap_state->lookup_ret_N10_N15;
    meth_N11_N16 = ycf_my_trap_state->meth_N11_N16;
    tb_N12_N17 = ycf_my_trap_state->tb_N12_N17;
    list_N13_N18 = ycf_my_trap_state->list_N13_N18;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_destroy_stack_allocator(&ycf_frame_alloc_data, ycf_yield_free, ycf_yield_alloc_free_context);
     ycf_yield_free(ycf_my_trap_state, ycf_yield_alloc_free_context);
     }
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  int  ets_insert_new_2_list_has_member_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context){
     struct gen_ycf_trap_state_for_ets_insert_new_2_list_has_member* ycf_my_trap_state = *ycf_trap_state;
return ets_insert_new_2_list_has_member_ycf_gen_yielding(ycf_number_of_reduction_param,
                    ycf_trap_state,
                    ycf_extra_context,
                    ycf_my_trap_state->ycf_yield_alloc,
                    ycf_my_trap_state->ycf_yield_free,
                    ycf_my_trap_state->ycf_yield_alloc_free_context,
                    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size,
                    ycf_my_trap_state->ycf_stack_alloc_data
                    ,ycf_my_trap_state->tb_N12_N17,ycf_my_trap_state->list_N13_N18)
;
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

#define YCF_IN_YIELDING_FUN 1
#undef YCF_STACK_ALLOC
#define YCF_STACK_ALLOC(SIZE) \
    ycf_stack_alloc(SIZE,\
                    &ycf_frame_alloc_data,\
                    ycf_yield_alloc, ycf_yield_alloc_free_context,\
                     ycf_stack_alloc_size_or_max_size)

/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(uninitialized)
/* clang-format on */


static int ets_insert_new_2_list_has_member_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,DbTable* tb_N12_N17, Eterm list_N13_N18)
{
    Eterm lst_N9_N14;
    Eterm lookup_ret_N10_N15;
    DbTableMethod* meth_N11_N16;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;

{
  
  ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
  ycf_frame_alloc_data.size = 0;
  ycf_frame_alloc_data.max_size = ycf_stack_alloc_size_or_max_size;
  ycf_frame_alloc_data.data = ycf_stack_alloc_data;
  ycf_frame_alloc_data.needs_freeing = 0;
  if(*ycf_trap_state != NULL){
     struct gen_ycf_trap_state_for_ets_insert_new_2_list_has_member* ycf_my_trap_state = *ycf_trap_state;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    lst_N9_N14 = ycf_my_trap_state->lst_N9_N14;
    lookup_ret_N10_N15 = ycf_my_trap_state->lookup_ret_N10_N15;
    meth_N11_N16 = ycf_my_trap_state->meth_N11_N16;
    tb_N12_N17 = ycf_my_trap_state->tb_N12_N17;
    list_N13_N18 = ycf_my_trap_state->list_N13_N18;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
     switch(ycf_trap_location){
case 1: goto ycf_yield_location_label_1;

}
  }
}
{

}
    meth_N11_N16 = tb_N12_N17->common.meth;
    for (lst_N9_N14 = list_N13_N18; is_list(lst_N9_N14); lst_N9_N14 = CDR(list_val(lst_N9_N14)))
{
{
ycf_nr_of_reductions = ycf_nr_of_reductions - (1);
if (ycf_nr_of_reductions <= 0) {
  ycf_trap_location = 1;
  goto ycf_do_yield_label_ets_insert_new_2_list_has_member;
  ycf_yield_location_label_1:;
}

}
    {
        meth_N11_N16->db_member(tb_N12_N17,
                        TERM_GETKEY(tb_N12_N17,CAR(list_val(lst_N9_N14))),
                        &lookup_ret_N10_N15);
        if (lookup_ret_N10_N15 != am_false)
{
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

            return 1;
}

}
    }

}
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

    return 0;
}

{
  struct gen_ycf_trap_state_for_ets_insert_new_2_list_has_member* ycf_my_trap_state;
  ycf_do_yield_label_ets_insert_new_2_list_has_member:;
  
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */
  if (*ycf_trap_state == NULL) {
    ycf_my_trap_state = ycf_yield_alloc(sizeof(struct gen_ycf_trap_state_for_ets_insert_new_2_list_has_member), ycf_yield_alloc_free_context);
      } else {
    ycf_my_trap_state = *ycf_trap_state;
  }
      ycf_my_trap_state->ycf_yield_alloc = ycf_yield_alloc;
    ycf_my_trap_state->ycf_yield_free = ycf_yield_free;
    ycf_my_trap_state->ycf_yield_alloc_free_context = ycf_yield_alloc_free_context;
    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size = ycf_stack_alloc_size_or_max_size;
    ycf_my_trap_state->ycf_stack_alloc_data = ycf_stack_alloc_data;
    ycf_my_trap_state->ycf_trap_location = ycf_trap_location;
    ycf_my_trap_state->ycf_nr_of_reductions = ycf_nr_of_reductions;
    ycf_my_trap_state->ycf_frame_alloc_data = ycf_frame_alloc_data;
    ycf_my_trap_state->lst_N9_N14 = lst_N9_N14;
    ycf_my_trap_state->lookup_ret_N10_N15 = lookup_ret_N10_N15;
    ycf_my_trap_state->meth_N11_N16 = meth_N11_N16;
    ycf_my_trap_state->tb_N12_N17 = tb_N12_N17;
    ycf_my_trap_state->list_N13_N18 = list_N13_N18;

  *ycf_nr_of_reductions_param = ycf_nr_of_reductions;
  *ycf_trap_state = ycf_my_trap_state;
      {
    static   int  const ycf_unused_ret_value;
    return ycf_unused_ret_value;
  }

}
}
/* clang-format off */
YCF_GCC_DIAG_ON(uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */

#undef YCF_STACK_ALLOC
#undef YCF_IN_YIELDING_FUN



struct gen_ycf_trap_state_for_ets_insert_2_list_from_p_heap {
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    Eterm lst_N19_N24;
    DbTableMethod* meth_N20_N25;
    int cret_N21_N26;DbTable* tb_N22_N27; Eterm list_N23_N28;
};


/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_2_list_from_p_heap_ycf_gen_destroy(void* ycf_my_trap_state_param){
     {
     struct gen_ycf_trap_state_for_ets_insert_2_list_from_p_heap* ycf_my_trap_state = ycf_my_trap_state_param;
     
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    Eterm lst_N19_N24;
    DbTableMethod* meth_N20_N25;
    int cret_N21_N26;DbTable* tb_N22_N27; Eterm list_N23_N28;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    lst_N19_N24 = ycf_my_trap_state->lst_N19_N24;
    meth_N20_N25 = ycf_my_trap_state->meth_N20_N25;
    cret_N21_N26 = ycf_my_trap_state->cret_N21_N26;
    tb_N22_N27 = ycf_my_trap_state->tb_N22_N27;
    list_N23_N28 = ycf_my_trap_state->list_N23_N28;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_destroy_stack_allocator(&ycf_frame_alloc_data, ycf_yield_free, ycf_yield_alloc_free_context);
     ycf_yield_free(ycf_my_trap_state, ycf_yield_alloc_free_context);
     }
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  int  ets_insert_2_list_from_p_heap_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context){
     struct gen_ycf_trap_state_for_ets_insert_2_list_from_p_heap* ycf_my_trap_state = *ycf_trap_state;
return ets_insert_2_list_from_p_heap_ycf_gen_yielding(ycf_number_of_reduction_param,
                    ycf_trap_state,
                    ycf_extra_context,
                    ycf_my_trap_state->ycf_yield_alloc,
                    ycf_my_trap_state->ycf_yield_free,
                    ycf_my_trap_state->ycf_yield_alloc_free_context,
                    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size,
                    ycf_my_trap_state->ycf_stack_alloc_data
                    ,ycf_my_trap_state->tb_N22_N27,ycf_my_trap_state->list_N23_N28)
;
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

#define YCF_IN_YIELDING_FUN 1
#undef YCF_STACK_ALLOC
#define YCF_STACK_ALLOC(SIZE) \
    ycf_stack_alloc(SIZE,\
                    &ycf_frame_alloc_data,\
                    ycf_yield_alloc, ycf_yield_alloc_free_context,\
                     ycf_stack_alloc_size_or_max_size)

/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(uninitialized)
/* clang-format on */


static int ets_insert_2_list_from_p_heap_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,DbTable* tb_N22_N27, Eterm list_N23_N28)
{
    Eterm lst_N19_N24;
    DbTableMethod* meth_N20_N25;
    int cret_N21_N26;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;

{
  
  ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
  ycf_frame_alloc_data.size = 0;
  ycf_frame_alloc_data.max_size = ycf_stack_alloc_size_or_max_size;
  ycf_frame_alloc_data.data = ycf_stack_alloc_data;
  ycf_frame_alloc_data.needs_freeing = 0;
  if(*ycf_trap_state != NULL){
     struct gen_ycf_trap_state_for_ets_insert_2_list_from_p_heap* ycf_my_trap_state = *ycf_trap_state;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    lst_N19_N24 = ycf_my_trap_state->lst_N19_N24;
    meth_N20_N25 = ycf_my_trap_state->meth_N20_N25;
    cret_N21_N26 = ycf_my_trap_state->cret_N21_N26;
    tb_N22_N27 = ycf_my_trap_state->tb_N22_N27;
    list_N23_N28 = ycf_my_trap_state->list_N23_N28;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
     switch(ycf_trap_location){
case 1: goto ycf_yield_location_label_1;
case 2: goto ycf_yield_location_label_2;

}
  }
}
{

}
    meth_N20_N25 = tb_N22_N27->common.meth;
    cret_N21_N26 = DB_ERROR_NONE;
    for (lst_N19_N24 = list_N23_N28; is_list(lst_N19_N24); lst_N19_N24 = CDR(list_val(lst_N19_N24)))
{
{
ycf_nr_of_reductions = ycf_nr_of_reductions - (1);
if (ycf_nr_of_reductions <= 0) {
  ycf_trap_location = 1;
  goto ycf_do_yield_label_ets_insert_2_list_from_p_heap;
  ycf_yield_location_label_1:;
}

}
    {
        LOCAL_VARIABLE(SWord, consumed_reds);
        consumed_reds = 1;
        cret_N21_N26 = meth_N20_N25->db_put(tb_N22_N27, CAR(list_val(lst_N19_N24)), 0, &consumed_reds);
        if (cret_N21_N26 != DB_ERROR_NONE)
{
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

   
            return cret_N21_N26;
}

}
{
ycf_nr_of_reductions = ycf_nr_of_reductions - (consumed_reds);
if (ycf_nr_of_reductions <= 0) {
  ycf_trap_location = 2;
  goto ycf_do_yield_label_ets_insert_2_list_from_p_heap;
  ycf_yield_location_label_2:;
}

}
    }

}
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

    return DB_ERROR_NONE;
}

{
  struct gen_ycf_trap_state_for_ets_insert_2_list_from_p_heap* ycf_my_trap_state;
  ycf_do_yield_label_ets_insert_2_list_from_p_heap:;
  
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */
  if (*ycf_trap_state == NULL) {
    ycf_my_trap_state = ycf_yield_alloc(sizeof(struct gen_ycf_trap_state_for_ets_insert_2_list_from_p_heap), ycf_yield_alloc_free_context);
      } else {
    ycf_my_trap_state = *ycf_trap_state;
  }
      ycf_my_trap_state->ycf_yield_alloc = ycf_yield_alloc;
    ycf_my_trap_state->ycf_yield_free = ycf_yield_free;
    ycf_my_trap_state->ycf_yield_alloc_free_context = ycf_yield_alloc_free_context;
    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size = ycf_stack_alloc_size_or_max_size;
    ycf_my_trap_state->ycf_stack_alloc_data = ycf_stack_alloc_data;
    ycf_my_trap_state->ycf_trap_location = ycf_trap_location;
    ycf_my_trap_state->ycf_nr_of_reductions = ycf_nr_of_reductions;
    ycf_my_trap_state->ycf_frame_alloc_data = ycf_frame_alloc_data;
    ycf_my_trap_state->lst_N19_N24 = lst_N19_N24;
    ycf_my_trap_state->meth_N20_N25 = meth_N20_N25;
    ycf_my_trap_state->cret_N21_N26 = cret_N21_N26;
    ycf_my_trap_state->tb_N22_N27 = tb_N22_N27;
    ycf_my_trap_state->list_N23_N28 = list_N23_N28;

  *ycf_nr_of_reductions_param = ycf_nr_of_reductions;
  *ycf_trap_state = ycf_my_trap_state;
      {
    static   int  const ycf_unused_ret_value;
    return ycf_unused_ret_value;
  }

}
}
/* clang-format off */
YCF_GCC_DIAG_ON(uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */

#undef YCF_STACK_ALLOC
#undef YCF_IN_YIELDING_FUN



struct gen_ycf_trap_state_for_ets_insert_2_list_destroy_copied_dbterms {
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    void* lst_N29_N34;
    void* term_N30_N35;DbTableMethod* meth_N31_N36;
                                                     int compressed_N32_N37;
                                                     void* db_term_list_N33_N38;
};


/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_2_list_destroy_copied_dbterms_ycf_gen_destroy(void* ycf_my_trap_state_param){
     {
     struct gen_ycf_trap_state_for_ets_insert_2_list_destroy_copied_dbterms* ycf_my_trap_state = ycf_my_trap_state_param;
     
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    void* lst_N29_N34;
    void* term_N30_N35;DbTableMethod* meth_N31_N36;
                                                     int compressed_N32_N37;
                                                     void* db_term_list_N33_N38;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    lst_N29_N34 = ycf_my_trap_state->lst_N29_N34;
    term_N30_N35 = ycf_my_trap_state->term_N30_N35;
    meth_N31_N36 = ycf_my_trap_state->meth_N31_N36;
    compressed_N32_N37 = ycf_my_trap_state->compressed_N32_N37;
    db_term_list_N33_N38 = ycf_my_trap_state->db_term_list_N33_N38;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_destroy_stack_allocator(&ycf_frame_alloc_data, ycf_yield_free, ycf_yield_alloc_free_context);
     ycf_yield_free(ycf_my_trap_state, ycf_yield_alloc_free_context);
     }
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  void  ets_insert_2_list_destroy_copied_dbterms_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context){
     struct gen_ycf_trap_state_for_ets_insert_2_list_destroy_copied_dbterms* ycf_my_trap_state = *ycf_trap_state;
ets_insert_2_list_destroy_copied_dbterms_ycf_gen_yielding(ycf_number_of_reduction_param,
                    ycf_trap_state,
                    ycf_extra_context,
                    ycf_my_trap_state->ycf_yield_alloc,
                    ycf_my_trap_state->ycf_yield_free,
                    ycf_my_trap_state->ycf_yield_alloc_free_context,
                    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size,
                    ycf_my_trap_state->ycf_stack_alloc_data
                    ,ycf_my_trap_state->meth_N31_N36,ycf_my_trap_state->compressed_N32_N37,ycf_my_trap_state->db_term_list_N33_N38)
;
return;
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

#define YCF_IN_YIELDING_FUN 1
#undef YCF_STACK_ALLOC
#define YCF_STACK_ALLOC(SIZE) \
    ycf_stack_alloc(SIZE,\
                    &ycf_frame_alloc_data,\
                    ycf_yield_alloc, ycf_yield_alloc_free_context,\
                     ycf_stack_alloc_size_or_max_size)

/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(uninitialized)
/* clang-format on */

/* This function is called both as is, and as YCF transformed. */
static void ets_insert_2_list_destroy_copied_dbterms_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,DbTableMethod* meth_N31_N36,
                                                     int compressed_N32_N37,
                                                     void* db_term_list_N33_N38)
{
    void* lst_N29_N34;
    void* term_N30_N35;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;

{
  
  ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
  ycf_frame_alloc_data.size = 0;
  ycf_frame_alloc_data.max_size = ycf_stack_alloc_size_or_max_size;
  ycf_frame_alloc_data.data = ycf_stack_alloc_data;
  ycf_frame_alloc_data.needs_freeing = 0;
  if(*ycf_trap_state != NULL){
     struct gen_ycf_trap_state_for_ets_insert_2_list_destroy_copied_dbterms* ycf_my_trap_state = *ycf_trap_state;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    lst_N29_N34 = ycf_my_trap_state->lst_N29_N34;
    term_N30_N35 = ycf_my_trap_state->term_N30_N35;
    meth_N31_N36 = ycf_my_trap_state->meth_N31_N36;
    compressed_N32_N37 = ycf_my_trap_state->compressed_N32_N37;
    db_term_list_N33_N38 = ycf_my_trap_state->db_term_list_N33_N38;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
     switch(ycf_trap_location){
case 1: goto ycf_yield_location_label_1;

}
  }
}
{

}
    lst_N29_N34 = db_term_list_N33_N38;
    term_N30_N35 = NULL;
    while (lst_N29_N34 != NULL)
{
{
ycf_nr_of_reductions = ycf_nr_of_reductions - (1);
if (ycf_nr_of_reductions <= 0) {
  ycf_trap_location = 1;
  goto ycf_do_yield_label_ets_insert_2_list_destroy_copied_dbterms;
  ycf_yield_location_label_1:;
}

}
    {
        term_N30_N35 = meth_N31_N36->db_dbterm_list_remove_first(&lst_N29_N34);
        meth_N31_N36->db_free_dbterm(compressed_N32_N37, term_N30_N35);
    }

}
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

return;
}
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

return;
}

{
  struct gen_ycf_trap_state_for_ets_insert_2_list_destroy_copied_dbterms* ycf_my_trap_state;
  ycf_do_yield_label_ets_insert_2_list_destroy_copied_dbterms:;
  
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */
  if (*ycf_trap_state == NULL) {
    ycf_my_trap_state = ycf_yield_alloc(sizeof(struct gen_ycf_trap_state_for_ets_insert_2_list_destroy_copied_dbterms), ycf_yield_alloc_free_context);
      } else {
    ycf_my_trap_state = *ycf_trap_state;
  }
      ycf_my_trap_state->ycf_yield_alloc = ycf_yield_alloc;
    ycf_my_trap_state->ycf_yield_free = ycf_yield_free;
    ycf_my_trap_state->ycf_yield_alloc_free_context = ycf_yield_alloc_free_context;
    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size = ycf_stack_alloc_size_or_max_size;
    ycf_my_trap_state->ycf_stack_alloc_data = ycf_stack_alloc_data;
    ycf_my_trap_state->ycf_trap_location = ycf_trap_location;
    ycf_my_trap_state->ycf_nr_of_reductions = ycf_nr_of_reductions;
    ycf_my_trap_state->ycf_frame_alloc_data = ycf_frame_alloc_data;
    ycf_my_trap_state->lst_N29_N34 = lst_N29_N34;
    ycf_my_trap_state->term_N30_N35 = term_N30_N35;
    ycf_my_trap_state->meth_N31_N36 = meth_N31_N36;
    ycf_my_trap_state->compressed_N32_N37 = compressed_N32_N37;
    ycf_my_trap_state->db_term_list_N33_N38 = db_term_list_N33_N38;

  *ycf_nr_of_reductions_param = ycf_nr_of_reductions;
  *ycf_trap_state = ycf_my_trap_state;
    return;

}
}
/* clang-format off */
YCF_GCC_DIAG_ON(uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */

#undef YCF_STACK_ALLOC
#undef YCF_IN_YIELDING_FUN



struct gen_ycf_trap_state_for_ets_insert_2_list_copy_term_list {
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    void* db_term_list_N39_N46;
    void *term_N40_N47;
    Eterm lst_N41_N48;DbTableMethod* meth_N42_N49;
                                              int compress_N43_N50;
                                              int keypos_N44_N51;
                                              Eterm list_N45_N52;
};


/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_2_list_copy_term_list_ycf_gen_destroy(void* ycf_my_trap_state_param){
     {
     struct gen_ycf_trap_state_for_ets_insert_2_list_copy_term_list* ycf_my_trap_state = ycf_my_trap_state_param;
     
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    void* db_term_list_N39_N46;
    void *term_N40_N47;
    Eterm lst_N41_N48;DbTableMethod* meth_N42_N49;
                                              int compress_N43_N50;
                                              int keypos_N44_N51;
                                              Eterm list_N45_N52;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    db_term_list_N39_N46 = ycf_my_trap_state->db_term_list_N39_N46;
    term_N40_N47 = ycf_my_trap_state->term_N40_N47;
    lst_N41_N48 = ycf_my_trap_state->lst_N41_N48;
    meth_N42_N49 = ycf_my_trap_state->meth_N42_N49;
    compress_N43_N50 = ycf_my_trap_state->compress_N43_N50;
    keypos_N44_N51 = ycf_my_trap_state->keypos_N44_N51;
    list_N45_N52 = ycf_my_trap_state->list_N45_N52;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     
/* YCF SPECIAL CUSTOM CODE START */
{
 {
        ets_insert_2_list_destroy_copied_dbterms(meth_N42_N49,
                                                 compress_N43_N50,
                                                 db_term_list_N39_N46);
    }}

/* YCF SPECIAL CUSTOM CODE END */

     ycf_destroy_stack_allocator(&ycf_frame_alloc_data, ycf_yield_free, ycf_yield_alloc_free_context);
     ycf_yield_free(ycf_my_trap_state, ycf_yield_alloc_free_context);
     }
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

    static  void  *  ets_insert_2_list_copy_term_list_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context){
     struct gen_ycf_trap_state_for_ets_insert_2_list_copy_term_list* ycf_my_trap_state = *ycf_trap_state;
return ets_insert_2_list_copy_term_list_ycf_gen_yielding(ycf_number_of_reduction_param,
                    ycf_trap_state,
                    ycf_extra_context,
                    ycf_my_trap_state->ycf_yield_alloc,
                    ycf_my_trap_state->ycf_yield_free,
                    ycf_my_trap_state->ycf_yield_alloc_free_context,
                    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size,
                    ycf_my_trap_state->ycf_stack_alloc_data
                    ,ycf_my_trap_state->meth_N42_N49,ycf_my_trap_state->compress_N43_N50,ycf_my_trap_state->keypos_N44_N51,ycf_my_trap_state->list_N45_N52)
;
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

#define YCF_IN_YIELDING_FUN 1
#undef YCF_STACK_ALLOC
#define YCF_STACK_ALLOC(SIZE) \
    ycf_stack_alloc(SIZE,\
                    &ycf_frame_alloc_data,\
                    ycf_yield_alloc, ycf_yield_alloc_free_context,\
                     ycf_stack_alloc_size_or_max_size)

/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(uninitialized)
/* clang-format on */
static void* ets_insert_2_list_copy_term_list_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,DbTableMethod* meth_N42_N49,
                                              int compress_N43_N50,
                                              int keypos_N44_N51,
                                              Eterm list_N45_N52)
{
    void* db_term_list_N39_N46;
    void *term_N40_N47;
    Eterm lst_N41_N48;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;

{
  
  ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
  ycf_frame_alloc_data.size = 0;
  ycf_frame_alloc_data.max_size = ycf_stack_alloc_size_or_max_size;
  ycf_frame_alloc_data.data = ycf_stack_alloc_data;
  ycf_frame_alloc_data.needs_freeing = 0;
  if(*ycf_trap_state != NULL){
     struct gen_ycf_trap_state_for_ets_insert_2_list_copy_term_list* ycf_my_trap_state = *ycf_trap_state;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    db_term_list_N39_N46 = ycf_my_trap_state->db_term_list_N39_N46;
    term_N40_N47 = ycf_my_trap_state->term_N40_N47;
    lst_N41_N48 = ycf_my_trap_state->lst_N41_N48;
    meth_N42_N49 = ycf_my_trap_state->meth_N42_N49;
    compress_N43_N50 = ycf_my_trap_state->compress_N43_N50;
    keypos_N44_N51 = ycf_my_trap_state->keypos_N44_N51;
    list_N45_N52 = ycf_my_trap_state->list_N45_N52;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
     switch(ycf_trap_location){
case 1: goto ycf_yield_location_label_1;

}
  }
}
{

}
    db_term_list_N39_N46 = NULL;
    for (lst_N41_N48 = list_N45_N52; is_list(lst_N41_N48); lst_N41_N48 = CDR(list_val(lst_N41_N48)))
{
{
ycf_nr_of_reductions = ycf_nr_of_reductions - (1);
if (ycf_nr_of_reductions <= 0) {
  ycf_trap_location = 1;
  goto ycf_do_yield_label_ets_insert_2_list_copy_term_list;
  ycf_yield_location_label_1:;
}

}
    {
        term_N40_N47 = meth_N42_N49->db_eterm_to_dbterm(compress_N43_N50,
                                        keypos_N44_N51,
                                        CAR(list_val(lst_N41_N48)));
        if (db_term_list_N39_N46 != NULL)
{
    {
            db_term_list_N39_N46 =
                meth_N42_N49->db_dbterm_list_prepend(db_term_list_N39_N46,
                                             term_N40_N47);
        }

} else
{
    {
            db_term_list_N39_N46 = term_N40_N47;
        }

}
    }

}
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */


    return db_term_list_N39_N46;
}
/* YCF Replaced special code */


{
  struct gen_ycf_trap_state_for_ets_insert_2_list_copy_term_list* ycf_my_trap_state;
  ycf_do_yield_label_ets_insert_2_list_copy_term_list:;
  
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */
  if (*ycf_trap_state == NULL) {
    ycf_my_trap_state = ycf_yield_alloc(sizeof(struct gen_ycf_trap_state_for_ets_insert_2_list_copy_term_list), ycf_yield_alloc_free_context);
      } else {
    ycf_my_trap_state = *ycf_trap_state;
  }
      ycf_my_trap_state->ycf_yield_alloc = ycf_yield_alloc;
    ycf_my_trap_state->ycf_yield_free = ycf_yield_free;
    ycf_my_trap_state->ycf_yield_alloc_free_context = ycf_yield_alloc_free_context;
    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size = ycf_stack_alloc_size_or_max_size;
    ycf_my_trap_state->ycf_stack_alloc_data = ycf_stack_alloc_data;
    ycf_my_trap_state->ycf_trap_location = ycf_trap_location;
    ycf_my_trap_state->ycf_nr_of_reductions = ycf_nr_of_reductions;
    ycf_my_trap_state->ycf_frame_alloc_data = ycf_frame_alloc_data;
    ycf_my_trap_state->db_term_list_N39_N46 = db_term_list_N39_N46;
    ycf_my_trap_state->term_N40_N47 = term_N40_N47;
    ycf_my_trap_state->lst_N41_N48 = lst_N41_N48;
    ycf_my_trap_state->meth_N42_N49 = meth_N42_N49;
    ycf_my_trap_state->compress_N43_N50 = compress_N43_N50;
    ycf_my_trap_state->keypos_N44_N51 = keypos_N44_N51;
    ycf_my_trap_state->list_N45_N52 = list_N45_N52;

  *ycf_nr_of_reductions_param = ycf_nr_of_reductions;
  *ycf_trap_state = ycf_my_trap_state;
      {
    static    void  *  const ycf_unused_ret_value;
    return ycf_unused_ret_value;
  }

}

}
/* clang-format off */
YCF_GCC_DIAG_ON(uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */

#undef YCF_STACK_ALLOC
#undef YCF_IN_YIELDING_FUN



struct gen_ycf_trap_state_for_ets_insert_new_2_dbterm_list_has_member {
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    Eterm lookup_ret_N53_N60;
    DbTableMethod* meth_N54_N61;
    void* lst_N55_N62;
    void* term_N56_N63;
    Eterm key_N57_N64;DbTable* tb_N58_N65; void* db_term_list_N59_N66;
};


/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_new_2_dbterm_list_has_member_ycf_gen_destroy(void* ycf_my_trap_state_param){
     {
     struct gen_ycf_trap_state_for_ets_insert_new_2_dbterm_list_has_member* ycf_my_trap_state = ycf_my_trap_state_param;
     
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    Eterm lookup_ret_N53_N60;
    DbTableMethod* meth_N54_N61;
    void* lst_N55_N62;
    void* term_N56_N63;
    Eterm key_N57_N64;DbTable* tb_N58_N65; void* db_term_list_N59_N66;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    lookup_ret_N53_N60 = ycf_my_trap_state->lookup_ret_N53_N60;
    meth_N54_N61 = ycf_my_trap_state->meth_N54_N61;
    lst_N55_N62 = ycf_my_trap_state->lst_N55_N62;
    term_N56_N63 = ycf_my_trap_state->term_N56_N63;
    key_N57_N64 = ycf_my_trap_state->key_N57_N64;
    tb_N58_N65 = ycf_my_trap_state->tb_N58_N65;
    db_term_list_N59_N66 = ycf_my_trap_state->db_term_list_N59_N66;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_destroy_stack_allocator(&ycf_frame_alloc_data, ycf_yield_free, ycf_yield_alloc_free_context);
     ycf_yield_free(ycf_my_trap_state, ycf_yield_alloc_free_context);
     }
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  int  ets_insert_new_2_dbterm_list_has_member_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context){
     struct gen_ycf_trap_state_for_ets_insert_new_2_dbterm_list_has_member* ycf_my_trap_state = *ycf_trap_state;
return ets_insert_new_2_dbterm_list_has_member_ycf_gen_yielding(ycf_number_of_reduction_param,
                    ycf_trap_state,
                    ycf_extra_context,
                    ycf_my_trap_state->ycf_yield_alloc,
                    ycf_my_trap_state->ycf_yield_free,
                    ycf_my_trap_state->ycf_yield_alloc_free_context,
                    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size,
                    ycf_my_trap_state->ycf_stack_alloc_data
                    ,ycf_my_trap_state->tb_N58_N65,ycf_my_trap_state->db_term_list_N59_N66)
;
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

#define YCF_IN_YIELDING_FUN 1
#undef YCF_STACK_ALLOC
#define YCF_STACK_ALLOC(SIZE) \
    ycf_stack_alloc(SIZE,\
                    &ycf_frame_alloc_data,\
                    ycf_yield_alloc, ycf_yield_alloc_free_context,\
                     ycf_stack_alloc_size_or_max_size)

/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(uninitialized)
/* clang-format on */


static int ets_insert_new_2_dbterm_list_has_member_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,DbTable* tb_N58_N65, void* db_term_list_N59_N66)
{
    Eterm lookup_ret_N53_N60;
    DbTableMethod* meth_N54_N61;
    void* lst_N55_N62;
    void* term_N56_N63;
    Eterm key_N57_N64;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;

{
  
  ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
  ycf_frame_alloc_data.size = 0;
  ycf_frame_alloc_data.max_size = ycf_stack_alloc_size_or_max_size;
  ycf_frame_alloc_data.data = ycf_stack_alloc_data;
  ycf_frame_alloc_data.needs_freeing = 0;
  if(*ycf_trap_state != NULL){
     struct gen_ycf_trap_state_for_ets_insert_new_2_dbterm_list_has_member* ycf_my_trap_state = *ycf_trap_state;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    lookup_ret_N53_N60 = ycf_my_trap_state->lookup_ret_N53_N60;
    meth_N54_N61 = ycf_my_trap_state->meth_N54_N61;
    lst_N55_N62 = ycf_my_trap_state->lst_N55_N62;
    term_N56_N63 = ycf_my_trap_state->term_N56_N63;
    key_N57_N64 = ycf_my_trap_state->key_N57_N64;
    tb_N58_N65 = ycf_my_trap_state->tb_N58_N65;
    db_term_list_N59_N66 = ycf_my_trap_state->db_term_list_N59_N66;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
     switch(ycf_trap_location){
case 1: goto ycf_yield_location_label_1;

}
  }
}
{

}
    meth_N54_N61 = tb_N58_N65->common.meth;
    lst_N55_N62 = db_term_list_N59_N66;
    term_N56_N63 = NULL;
    while (lst_N55_N62 != NULL)
{
{
ycf_nr_of_reductions = ycf_nr_of_reductions - (1);
if (ycf_nr_of_reductions <= 0) {
  ycf_trap_location = 1;
  goto ycf_do_yield_label_ets_insert_new_2_dbterm_list_has_member;
  ycf_yield_location_label_1:;
}

}
    {
        term_N56_N63 = meth_N54_N61->db_dbterm_list_remove_first(&lst_N55_N62);
        key_N57_N64 = meth_N54_N61->db_get_dbterm_key(tb_N58_N65, term_N56_N63);
        meth_N54_N61->db_member(tb_N58_N65, key_N57_N64, &lookup_ret_N53_N60);
        if (lookup_ret_N53_N60 != am_false)
{
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

            return 1;
}

}
    }

}
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

    return 0;
}

{
  struct gen_ycf_trap_state_for_ets_insert_new_2_dbterm_list_has_member* ycf_my_trap_state;
  ycf_do_yield_label_ets_insert_new_2_dbterm_list_has_member:;
  
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */
  if (*ycf_trap_state == NULL) {
    ycf_my_trap_state = ycf_yield_alloc(sizeof(struct gen_ycf_trap_state_for_ets_insert_new_2_dbterm_list_has_member), ycf_yield_alloc_free_context);
      } else {
    ycf_my_trap_state = *ycf_trap_state;
  }
      ycf_my_trap_state->ycf_yield_alloc = ycf_yield_alloc;
    ycf_my_trap_state->ycf_yield_free = ycf_yield_free;
    ycf_my_trap_state->ycf_yield_alloc_free_context = ycf_yield_alloc_free_context;
    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size = ycf_stack_alloc_size_or_max_size;
    ycf_my_trap_state->ycf_stack_alloc_data = ycf_stack_alloc_data;
    ycf_my_trap_state->ycf_trap_location = ycf_trap_location;
    ycf_my_trap_state->ycf_nr_of_reductions = ycf_nr_of_reductions;
    ycf_my_trap_state->ycf_frame_alloc_data = ycf_frame_alloc_data;
    ycf_my_trap_state->lookup_ret_N53_N60 = lookup_ret_N53_N60;
    ycf_my_trap_state->meth_N54_N61 = meth_N54_N61;
    ycf_my_trap_state->lst_N55_N62 = lst_N55_N62;
    ycf_my_trap_state->term_N56_N63 = term_N56_N63;
    ycf_my_trap_state->key_N57_N64 = key_N57_N64;
    ycf_my_trap_state->tb_N58_N65 = tb_N58_N65;
    ycf_my_trap_state->db_term_list_N59_N66 = db_term_list_N59_N66;

  *ycf_nr_of_reductions_param = ycf_nr_of_reductions;
  *ycf_trap_state = ycf_my_trap_state;
      {
    static   int  const ycf_unused_ret_value;
    return ycf_unused_ret_value;
  }

}
}
/* clang-format off */
YCF_GCC_DIAG_ON(uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */

#undef YCF_STACK_ALLOC
#undef YCF_IN_YIELDING_FUN



struct gen_ycf_trap_state_for_ets_insert_2_list_insert_db_term_list {
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    void* lst_N67_N72;
    void* term_N68_N73;
    DbTableMethod* meth_N69_N74;DbTable* tb_N70_N75;
                                                  void* list_N71_N76;
};


/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_2_list_insert_db_term_list_ycf_gen_destroy(void* ycf_my_trap_state_param){
     {
     struct gen_ycf_trap_state_for_ets_insert_2_list_insert_db_term_list* ycf_my_trap_state = ycf_my_trap_state_param;
     
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    void* lst_N67_N72;
    void* term_N68_N73;
    DbTableMethod* meth_N69_N74;DbTable* tb_N70_N75;
                                                  void* list_N71_N76;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    lst_N67_N72 = ycf_my_trap_state->lst_N67_N72;
    term_N68_N73 = ycf_my_trap_state->term_N68_N73;
    meth_N69_N74 = ycf_my_trap_state->meth_N69_N74;
    tb_N70_N75 = ycf_my_trap_state->tb_N70_N75;
    list_N71_N76 = ycf_my_trap_state->list_N71_N76;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_destroy_stack_allocator(&ycf_frame_alloc_data, ycf_yield_free, ycf_yield_alloc_free_context);
     ycf_yield_free(ycf_my_trap_state, ycf_yield_alloc_free_context);
     }
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  void  ets_insert_2_list_insert_db_term_list_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context){
     struct gen_ycf_trap_state_for_ets_insert_2_list_insert_db_term_list* ycf_my_trap_state = *ycf_trap_state;
ets_insert_2_list_insert_db_term_list_ycf_gen_yielding(ycf_number_of_reduction_param,
                    ycf_trap_state,
                    ycf_extra_context,
                    ycf_my_trap_state->ycf_yield_alloc,
                    ycf_my_trap_state->ycf_yield_free,
                    ycf_my_trap_state->ycf_yield_alloc_free_context,
                    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size,
                    ycf_my_trap_state->ycf_stack_alloc_data
                    ,ycf_my_trap_state->tb_N70_N75,ycf_my_trap_state->list_N71_N76)
;
return;
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

#define YCF_IN_YIELDING_FUN 1
#undef YCF_STACK_ALLOC
#define YCF_STACK_ALLOC(SIZE) \
    ycf_stack_alloc(SIZE,\
                    &ycf_frame_alloc_data,\
                    ycf_yield_alloc, ycf_yield_alloc_free_context,\
                     ycf_stack_alloc_size_or_max_size)

/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(uninitialized)
/* clang-format on */


static void ets_insert_2_list_insert_db_term_list_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,DbTable* tb_N70_N75,
                                                  void* list_N71_N76)
{
    void* lst_N67_N72;
    void* term_N68_N73;
    DbTableMethod* meth_N69_N74;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;

{
  
  ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
  ycf_frame_alloc_data.size = 0;
  ycf_frame_alloc_data.max_size = ycf_stack_alloc_size_or_max_size;
  ycf_frame_alloc_data.data = ycf_stack_alloc_data;
  ycf_frame_alloc_data.needs_freeing = 0;
  if(*ycf_trap_state != NULL){
     struct gen_ycf_trap_state_for_ets_insert_2_list_insert_db_term_list* ycf_my_trap_state = *ycf_trap_state;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    lst_N67_N72 = ycf_my_trap_state->lst_N67_N72;
    term_N68_N73 = ycf_my_trap_state->term_N68_N73;
    meth_N69_N74 = ycf_my_trap_state->meth_N69_N74;
    tb_N70_N75 = ycf_my_trap_state->tb_N70_N75;
    list_N71_N76 = ycf_my_trap_state->list_N71_N76;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
     switch(ycf_trap_location){
case 1: goto ycf_yield_location_label_1;
case 2: goto ycf_yield_location_label_2;

}
  }
}
{

}
    lst_N67_N72 = list_N71_N76;
    term_N68_N73 = NULL;
    meth_N69_N74 = tb_N70_N75->common.meth;
    do
{
{
ycf_nr_of_reductions = ycf_nr_of_reductions - (1);
if (ycf_nr_of_reductions <= 0) {
  ycf_trap_location = 1;
  goto ycf_do_yield_label_ets_insert_2_list_insert_db_term_list;
  ycf_yield_location_label_1:;
}

}
    {
        LOCAL_VARIABLE(SWord, consumed_reds);
        consumed_reds = 1;
        term_N68_N73 = meth_N69_N74->db_dbterm_list_remove_first(&lst_N67_N72);
        meth_N69_N74->db_put_dbterm(tb_N70_N75, term_N68_N73, 0, &consumed_reds);
{
ycf_nr_of_reductions = ycf_nr_of_reductions - (consumed_reds);
if (ycf_nr_of_reductions <= 0) {
  ycf_trap_location = 2;
  goto ycf_do_yield_label_ets_insert_2_list_insert_db_term_list;
  ycf_yield_location_label_2:;
}

}
    }

} while (lst_N67_N72 != NULL);
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

    return;
}
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

return;
}
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

return;
}

{
  struct gen_ycf_trap_state_for_ets_insert_2_list_insert_db_term_list* ycf_my_trap_state;
  ycf_do_yield_label_ets_insert_2_list_insert_db_term_list:;
  
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */
  if (*ycf_trap_state == NULL) {
    ycf_my_trap_state = ycf_yield_alloc(sizeof(struct gen_ycf_trap_state_for_ets_insert_2_list_insert_db_term_list), ycf_yield_alloc_free_context);
      } else {
    ycf_my_trap_state = *ycf_trap_state;
  }
      ycf_my_trap_state->ycf_yield_alloc = ycf_yield_alloc;
    ycf_my_trap_state->ycf_yield_free = ycf_yield_free;
    ycf_my_trap_state->ycf_yield_alloc_free_context = ycf_yield_alloc_free_context;
    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size = ycf_stack_alloc_size_or_max_size;
    ycf_my_trap_state->ycf_stack_alloc_data = ycf_stack_alloc_data;
    ycf_my_trap_state->ycf_trap_location = ycf_trap_location;
    ycf_my_trap_state->ycf_nr_of_reductions = ycf_nr_of_reductions;
    ycf_my_trap_state->ycf_frame_alloc_data = ycf_frame_alloc_data;
    ycf_my_trap_state->lst_N67_N72 = lst_N67_N72;
    ycf_my_trap_state->term_N68_N73 = term_N68_N73;
    ycf_my_trap_state->meth_N69_N74 = meth_N69_N74;
    ycf_my_trap_state->tb_N70_N75 = tb_N70_N75;
    ycf_my_trap_state->list_N71_N76 = list_N71_N76;

  *ycf_nr_of_reductions_param = ycf_nr_of_reductions;
  *ycf_trap_state = ycf_my_trap_state;
    return;

}
}
/* clang-format off */
YCF_GCC_DIAG_ON(uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */

#undef YCF_STACK_ALLOC
#undef YCF_IN_YIELDING_FUN



struct gen_ycf_trap_state_for_ets_insert_2_list {
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    int cret_N80_N107;
    void* db_term_list_N81_N108; /* OBS: memory managements depends on that
                                  db_term_list is initialized to NULL */
    DbTableMethod* meth_N82_N109;
    int compressed_N83_N110;
    int keypos_N84_N111;
    Uint32 tb_type_N85_N112;
    Uint bif_ix_N86_N113;
    long list_len_N87_N114;
        Eterm ret_N77_N115;
        long reds_boost_N78_N116;
            int* result_ptr_N79_N117;
void* ycf_sub_fun_trap_state_wb_N93;

void* ycf_sub_fun_trap_state_wb_N94;

void* ycf_sub_fun_trap_state_wb_N95;
int ycf_gen_control_tmp_N98;

void* ycf_sub_fun_trap_state_wb_N96;

void* ycf_sub_fun_trap_state_wb_N97;

void* ycf_sub_fun_trap_state_wb_N99;
void* ycf_sub_fun_trap_state_wb_N100;
void* ycf_sub_fun_trap_state_wb_N101;
int ycf_gen_control_tmp_N105;

void* ycf_sub_fun_trap_state_wb_N102;

void* ycf_sub_fun_trap_state_wb_N103;

void* ycf_sub_fun_trap_state_wb_N104;

void* ycf_sub_fun_trap_state_wb_N106;Process* p_N88_N118;
                                     Eterm table_id_N89_N119;
                                     DbTable *tb_N90_N120;
                                     Eterm list_N91_N121;
                                     int is_insert_new_N92_N122;
};


/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_2_list_ycf_gen_destroy(void* ycf_my_trap_state_param){
     {
     struct gen_ycf_trap_state_for_ets_insert_2_list* ycf_my_trap_state = ycf_my_trap_state_param;
     
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    int cret_N80_N107;
    void* db_term_list_N81_N108; /* OBS: memory managements depends on that
                                  db_term_list is initialized to NULL */
    DbTableMethod* meth_N82_N109;
    int compressed_N83_N110;
    int keypos_N84_N111;
    Uint32 tb_type_N85_N112;
    Uint bif_ix_N86_N113;
    long list_len_N87_N114;
        Eterm ret_N77_N115;
        long reds_boost_N78_N116;
            int* result_ptr_N79_N117;
void* ycf_sub_fun_trap_state_wb_N93;

void* ycf_sub_fun_trap_state_wb_N94;

void* ycf_sub_fun_trap_state_wb_N95;
int ycf_gen_control_tmp_N98;

void* ycf_sub_fun_trap_state_wb_N96;

void* ycf_sub_fun_trap_state_wb_N97;

void* ycf_sub_fun_trap_state_wb_N99;
void* ycf_sub_fun_trap_state_wb_N100;
void* ycf_sub_fun_trap_state_wb_N101;
int ycf_gen_control_tmp_N105;

void* ycf_sub_fun_trap_state_wb_N102;

void* ycf_sub_fun_trap_state_wb_N103;

void* ycf_sub_fun_trap_state_wb_N104;

void* ycf_sub_fun_trap_state_wb_N106;Process* p_N88_N118;
                                     Eterm table_id_N89_N119;
                                     DbTable *tb_N90_N120;
                                     Eterm list_N91_N121;
                                     int is_insert_new_N92_N122;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    cret_N80_N107 = ycf_my_trap_state->cret_N80_N107;
    db_term_list_N81_N108 = ycf_my_trap_state->db_term_list_N81_N108;
    meth_N82_N109 = ycf_my_trap_state->meth_N82_N109;
    compressed_N83_N110 = ycf_my_trap_state->compressed_N83_N110;
    keypos_N84_N111 = ycf_my_trap_state->keypos_N84_N111;
    tb_type_N85_N112 = ycf_my_trap_state->tb_type_N85_N112;
    bif_ix_N86_N113 = ycf_my_trap_state->bif_ix_N86_N113;
    list_len_N87_N114 = ycf_my_trap_state->list_len_N87_N114;
    ret_N77_N115 = ycf_my_trap_state->ret_N77_N115;
    reds_boost_N78_N116 = ycf_my_trap_state->reds_boost_N78_N116;
    result_ptr_N79_N117 = ycf_my_trap_state->result_ptr_N79_N117;
    ycf_sub_fun_trap_state_wb_N93 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N93;
    ycf_sub_fun_trap_state_wb_N94 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N94;
    ycf_sub_fun_trap_state_wb_N95 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N95;
    ycf_gen_control_tmp_N98 = ycf_my_trap_state->ycf_gen_control_tmp_N98;
    ycf_sub_fun_trap_state_wb_N96 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N96;
    ycf_sub_fun_trap_state_wb_N97 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N97;
    ycf_sub_fun_trap_state_wb_N99 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N99;
    ycf_sub_fun_trap_state_wb_N100 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N100;
    ycf_sub_fun_trap_state_wb_N101 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N101;
    ycf_gen_control_tmp_N105 = ycf_my_trap_state->ycf_gen_control_tmp_N105;
    ycf_sub_fun_trap_state_wb_N102 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N102;
    ycf_sub_fun_trap_state_wb_N103 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N103;
    ycf_sub_fun_trap_state_wb_N104 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N104;
    ycf_sub_fun_trap_state_wb_N106 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N106;
    p_N88_N118 = ycf_my_trap_state->p_N88_N118;
    table_id_N89_N119 = ycf_my_trap_state->table_id_N89_N119;
    tb_N90_N120 = ycf_my_trap_state->tb_N90_N120;
    list_N91_N121 = ycf_my_trap_state->list_N91_N121;
    is_insert_new_N92_N122 = ycf_my_trap_state->is_insert_new_N92_N122;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     
/* YCF SPECIAL CUSTOM CODE START */
{
  if(ycf_sub_fun_trap_state_wb_N93 != NULL)
{     ets_insert_2_list_check_ycf_gen_destroy(ycf_sub_fun_trap_state_wb_N93);
  }
}
{
  if(ycf_sub_fun_trap_state_wb_N94 != NULL)
{     ets_insert_2_list_lock_tbl_ycf_gen_destroy(ycf_sub_fun_trap_state_wb_N94);
  }
}
{
  if(ycf_sub_fun_trap_state_wb_N95 != NULL)
{     ets_insert_2_list_lock_tbl_ycf_gen_destroy(ycf_sub_fun_trap_state_wb_N95);
  }
}
{
  if(ycf_sub_fun_trap_state_wb_N96 != NULL)
{     ets_insert_new_2_list_has_member_ycf_gen_destroy(ycf_sub_fun_trap_state_wb_N96);
  }
}
{
  if(ycf_sub_fun_trap_state_wb_N97 != NULL)
{     ets_insert_2_list_from_p_heap_ycf_gen_destroy(ycf_sub_fun_trap_state_wb_N97);
  }
}
{
  if(ycf_sub_fun_trap_state_wb_N99 != NULL)
{     ets_insert_2_list_from_p_heap_ycf_gen_destroy(ycf_sub_fun_trap_state_wb_N99);
  }
}
{
  if(ycf_sub_fun_trap_state_wb_N100 != NULL)
{     ets_insert_2_list_copy_term_list_ycf_gen_destroy(ycf_sub_fun_trap_state_wb_N100);
  }
}
{
  if(ycf_sub_fun_trap_state_wb_N101 != NULL)
{     ets_insert_2_list_lock_tbl_ycf_gen_destroy(ycf_sub_fun_trap_state_wb_N101);
  }
}
{
  if(ycf_sub_fun_trap_state_wb_N102 != NULL)
{     ets_insert_new_2_dbterm_list_has_member_ycf_gen_destroy(ycf_sub_fun_trap_state_wb_N102);
  }
}
{
  if(ycf_sub_fun_trap_state_wb_N103 != NULL)
{     ets_insert_2_list_destroy_copied_dbterms_ycf_gen_destroy(ycf_sub_fun_trap_state_wb_N103);
  }
}
{
  if(ycf_sub_fun_trap_state_wb_N104 != NULL)
{     ets_insert_2_list_insert_db_term_list_ycf_gen_destroy(ycf_sub_fun_trap_state_wb_N104);
  }
}
{
  if(ycf_sub_fun_trap_state_wb_N106 != NULL)
{     ets_insert_2_list_insert_db_term_list_ycf_gen_destroy(ycf_sub_fun_trap_state_wb_N106);
  }
}
{
 {
        ets_insert_2_list_destroy_copied_dbterms(meth_N82_N109,
                                                 compressed_N83_N110,
                                                 db_term_list_N81_N108);
    }}

/* YCF SPECIAL CUSTOM CODE END */

     ycf_destroy_stack_allocator(&ycf_frame_alloc_data, ycf_yield_free, ycf_yield_alloc_free_context);
     ycf_yield_free(ycf_my_trap_state, ycf_yield_alloc_free_context);
     }
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  BIF_RETTYPE  ets_insert_2_list_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context){
     struct gen_ycf_trap_state_for_ets_insert_2_list* ycf_my_trap_state = *ycf_trap_state;
return ets_insert_2_list_ycf_gen_yielding(ycf_number_of_reduction_param,
                    ycf_trap_state,
                    ycf_extra_context,
                    ycf_my_trap_state->ycf_yield_alloc,
                    ycf_my_trap_state->ycf_yield_free,
                    ycf_my_trap_state->ycf_yield_alloc_free_context,
                    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size,
                    ycf_my_trap_state->ycf_stack_alloc_data
                    ,ycf_my_trap_state->p_N88_N118,ycf_my_trap_state->table_id_N89_N119,ycf_my_trap_state->tb_N90_N120,ycf_my_trap_state->list_N91_N121,ycf_my_trap_state->is_insert_new_N92_N122)
;
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

#define YCF_IN_YIELDING_FUN 1
#undef YCF_STACK_ALLOC
#define YCF_STACK_ALLOC(SIZE) \
    ycf_stack_alloc(SIZE,\
                    &ycf_frame_alloc_data,\
                    ycf_yield_alloc, ycf_yield_alloc_free_context,\
                     ycf_stack_alloc_size_or_max_size)

/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(uninitialized)
/* clang-format on */
static BIF_RETTYPE ets_insert_2_list_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,Process* p_N88_N118,
                                     Eterm table_id_N89_N119,
                                     DbTable *tb_N90_N120,
                                     Eterm list_N91_N121,
                                     int is_insert_new_N92_N122)
{
    int cret_N80_N107;
    void* db_term_list_N81_N108; /* OBS: memory managements depends on that
                                  db_term_list is initialized to NULL */
    DbTableMethod* meth_N82_N109;
    int compressed_N83_N110;
    int keypos_N84_N111;
    Uint32 tb_type_N85_N112;
    Uint bif_ix_N86_N113;
    long list_len_N87_N114;
        Eterm ret_N77_N115;
        long reds_boost_N78_N116;
            int* result_ptr_N79_N117;
void* ycf_sub_fun_trap_state_wb_N93;

void* ycf_sub_fun_trap_state_wb_N94;

void* ycf_sub_fun_trap_state_wb_N95;
int ycf_gen_control_tmp_N98;

void* ycf_sub_fun_trap_state_wb_N96;

void* ycf_sub_fun_trap_state_wb_N97;

void* ycf_sub_fun_trap_state_wb_N99;
void* ycf_sub_fun_trap_state_wb_N100;
void* ycf_sub_fun_trap_state_wb_N101;
int ycf_gen_control_tmp_N105;

void* ycf_sub_fun_trap_state_wb_N102;

void* ycf_sub_fun_trap_state_wb_N103;

void* ycf_sub_fun_trap_state_wb_N104;

void* ycf_sub_fun_trap_state_wb_N106;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;

{
  
  ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
  ycf_frame_alloc_data.size = 0;
  ycf_frame_alloc_data.max_size = ycf_stack_alloc_size_or_max_size;
  ycf_frame_alloc_data.data = ycf_stack_alloc_data;
  ycf_frame_alloc_data.needs_freeing = 0;
  if(*ycf_trap_state != NULL){
     struct gen_ycf_trap_state_for_ets_insert_2_list* ycf_my_trap_state = *ycf_trap_state;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    cret_N80_N107 = ycf_my_trap_state->cret_N80_N107;
    db_term_list_N81_N108 = ycf_my_trap_state->db_term_list_N81_N108;
    meth_N82_N109 = ycf_my_trap_state->meth_N82_N109;
    compressed_N83_N110 = ycf_my_trap_state->compressed_N83_N110;
    keypos_N84_N111 = ycf_my_trap_state->keypos_N84_N111;
    tb_type_N85_N112 = ycf_my_trap_state->tb_type_N85_N112;
    bif_ix_N86_N113 = ycf_my_trap_state->bif_ix_N86_N113;
    list_len_N87_N114 = ycf_my_trap_state->list_len_N87_N114;
    ret_N77_N115 = ycf_my_trap_state->ret_N77_N115;
    reds_boost_N78_N116 = ycf_my_trap_state->reds_boost_N78_N116;
    result_ptr_N79_N117 = ycf_my_trap_state->result_ptr_N79_N117;
    ycf_sub_fun_trap_state_wb_N93 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N93;
    ycf_sub_fun_trap_state_wb_N94 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N94;
    ycf_sub_fun_trap_state_wb_N95 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N95;
    ycf_gen_control_tmp_N98 = ycf_my_trap_state->ycf_gen_control_tmp_N98;
    ycf_sub_fun_trap_state_wb_N96 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N96;
    ycf_sub_fun_trap_state_wb_N97 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N97;
    ycf_sub_fun_trap_state_wb_N99 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N99;
    ycf_sub_fun_trap_state_wb_N100 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N100;
    ycf_sub_fun_trap_state_wb_N101 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N101;
    ycf_gen_control_tmp_N105 = ycf_my_trap_state->ycf_gen_control_tmp_N105;
    ycf_sub_fun_trap_state_wb_N102 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N102;
    ycf_sub_fun_trap_state_wb_N103 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N103;
    ycf_sub_fun_trap_state_wb_N104 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N104;
    ycf_sub_fun_trap_state_wb_N106 = ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N106;
    p_N88_N118 = ycf_my_trap_state->p_N88_N118;
    table_id_N89_N119 = ycf_my_trap_state->table_id_N89_N119;
    tb_N90_N120 = ycf_my_trap_state->tb_N90_N120;
    list_N91_N121 = ycf_my_trap_state->list_N91_N121;
    is_insert_new_N92_N122 = ycf_my_trap_state->is_insert_new_N92_N122;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
     switch(ycf_trap_location){
case 1: goto ycf_yield_location_label_1;
case 2: goto ycf_yield_location_label_2;
case 3: goto ycf_yield_location_label_3;
case 4: goto ycf_yield_location_label_4;
case 5: goto ycf_yield_location_label_5;
case 6: goto ycf_yield_location_label_6;
case 7: goto ycf_yield_location_label_7;
case 8: goto ycf_yield_location_label_8;
case 9: goto ycf_yield_location_label_9;
case 10: goto ycf_yield_location_label_10;
case 11: goto ycf_yield_location_label_11;
case 12: goto ycf_yield_location_label_12;

}
  }
}
{

ycf_sub_fun_trap_state_wb_N93 = NULL;

ycf_sub_fun_trap_state_wb_N94 = NULL;

ycf_sub_fun_trap_state_wb_N95 = NULL;

ycf_sub_fun_trap_state_wb_N96 = NULL;

ycf_sub_fun_trap_state_wb_N97 = NULL;

ycf_sub_fun_trap_state_wb_N99 = NULL;

ycf_sub_fun_trap_state_wb_N100 = NULL;

ycf_sub_fun_trap_state_wb_N101 = NULL;

ycf_sub_fun_trap_state_wb_N102 = NULL;

ycf_sub_fun_trap_state_wb_N103 = NULL;

ycf_sub_fun_trap_state_wb_N104 = NULL;

ycf_sub_fun_trap_state_wb_N106 = NULL;

}
    cret_N80_N107 = DB_ERROR_NONE;
    db_term_list_N81_N108 = NULL; /* OBS: memory managements depends on that
                                  db_term_list is initialized to NULL */
    meth_N82_N109 = tb_N90_N120->common.meth;
    compressed_N83_N110 = tb_N90_N120->common.compress;
    keypos_N84_N111 = tb_N90_N120->common.keypos;
    tb_type_N85_N112 = tb_N90_N120->common.type;
    bif_ix_N86_N113 = (is_insert_new_N92_N122 ? BIF_ets_insert_new_2 : BIF_ets_insert_2);
{

ycf_sub_fun_trap_state_wb_N93 = NULL;
{
  long  ycf_tmp_fun_call_tmp_var;
ycf_tmp_fun_call_tmp_var =  ets_insert_2_list_check_ycf_gen_yielding(&ycf_nr_of_reductions,
                         &ycf_sub_fun_trap_state_wb_N93,ycf_extra_context,
                         ycf_yield_alloc,ycf_yield_free,
                         ycf_yield_alloc_free_context,
                         YCF_ALLOC_NEXT_MAX_SIZE(),
                         YCF_ALLOC_NEXT_BLOCK()
                         ,keypos_N84_N111, list_N91_N121);
while(YCF_IS_YIELDED(ycf_sub_fun_trap_state_wb_N93)){
   ycf_trap_location = 1;
   goto ycf_do_yield_label_ets_insert_2_list;
  ycf_yield_location_label_1:;
   ycf_tmp_fun_call_tmp_var =  ets_insert_2_list_check_ycf_gen_continue(&ycf_nr_of_reductions,
                            &ycf_sub_fun_trap_state_wb_N93,
                            ycf_extra_context);
}

    /* tb should not be accessed after this point unless the table
       lock is held as the table can get deleted while the function is
       yielding */
    list_len_N87_N114 = ycf_tmp_fun_call_tmp_var;


}
/* YCF Replaced special code */

}

    if (list_len_N87_N114 < 0)
{
    {
{


ycf_sub_fun_trap_state_wb_N94 = NULL;
{
 ets_insert_2_list_lock_tbl_ycf_gen_yielding(&ycf_nr_of_reductions,
                         &ycf_sub_fun_trap_state_wb_N94,ycf_extra_context,
                         ycf_yield_alloc,ycf_yield_free,
                         ycf_yield_alloc_free_context,
                         YCF_ALLOC_NEXT_MAX_SIZE(),
                         YCF_ALLOC_NEXT_BLOCK()
                         ,table_id_N89_N119, p_N88_N118, bif_ix_N86_N113, ETS_INSERT_2_LIST_PROCESS_LOCAL);
while(YCF_IS_YIELDED(ycf_sub_fun_trap_state_wb_N94)){
   ycf_trap_location = 2;
   goto ycf_do_yield_label_ets_insert_2_list;
  ycf_yield_location_label_2:;
    ets_insert_2_list_lock_tbl_ycf_gen_continue(&ycf_nr_of_reductions,
                            &ycf_sub_fun_trap_state_wb_N94,
                            ycf_extra_context);
}


}
/* YCF Replaced special code */

}


        db_unlock(tb_N90_N120, LCK_WRITE);
        ERTS_BIF_PREP_ERROR_TRAPPED2(ret_N77_N115, p_N88_N118, BADARG, BIF_TRAP_EXPORT(bif_ix_N86_N113), table_id_N89_N119, list_N91_N121);
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

        return ret_N77_N115;
}
    }

}
    if (can_insert_without_yield(tb_type_N85_N112, list_len_N87_N114, YCF_NR_OF_REDS_LEFT()))
{
    {
{


ycf_sub_fun_trap_state_wb_N95 = NULL;
{
 ets_insert_2_list_lock_tbl_ycf_gen_yielding(&ycf_nr_of_reductions,
                         &ycf_sub_fun_trap_state_wb_N95,ycf_extra_context,
                         ycf_yield_alloc,ycf_yield_free,
                         ycf_yield_alloc_free_context,
                         YCF_ALLOC_NEXT_MAX_SIZE(),
                         YCF_ALLOC_NEXT_BLOCK()
                         ,table_id_N89_N119, p_N88_N118, bif_ix_N86_N113, ETS_INSERT_2_LIST_PROCESS_LOCAL);
while(YCF_IS_YIELDED(ycf_sub_fun_trap_state_wb_N95)){
   ycf_trap_location = 3;
   goto ycf_do_yield_label_ets_insert_2_list;
  ycf_yield_location_label_3:;
    ets_insert_2_list_lock_tbl_ycf_gen_continue(&ycf_nr_of_reductions,
                            &ycf_sub_fun_trap_state_wb_N95,
                            ycf_extra_context);
}


}
/* YCF Replaced special code */

}


        /* Ensure that we will not yield while inserting from heap */
        reds_boost_N78_N116 = YCF_MAX_NR_OF_REDS - YCF_NR_OF_REDS_LEFT();
        YCF_SET_NR_OF_REDS_LEFT(YCF_MAX_NR_OF_REDS);
        if (is_insert_new_N92_N122)
{
{
{


ycf_sub_fun_trap_state_wb_N96 = NULL;
{
  int  ycf_tmp_fun_call_tmp_var;
ycf_tmp_fun_call_tmp_var =     !  ! ets_insert_new_2_list_has_member_ycf_gen_yielding(&ycf_nr_of_reductions,
                         &ycf_sub_fun_trap_state_wb_N96,ycf_extra_context,
                         ycf_yield_alloc,ycf_yield_free,
                         ycf_yield_alloc_free_context,
                         YCF_ALLOC_NEXT_MAX_SIZE(),
                         YCF_ALLOC_NEXT_BLOCK()
                         ,tb_N90_N120, list_N91_N121);
while(YCF_IS_YIELDED(ycf_sub_fun_trap_state_wb_N96)){
   ycf_trap_location = 4;
   goto ycf_do_yield_label_ets_insert_2_list;
  ycf_yield_location_label_4:;
   ycf_tmp_fun_call_tmp_var =     !  ! ets_insert_new_2_list_has_member_ycf_gen_continue(&ycf_nr_of_reductions,
                            &ycf_sub_fun_trap_state_wb_N96,
                            ycf_extra_context);
}

ycf_gen_control_tmp_N98 = ycf_tmp_fun_call_tmp_var;


}
/* YCF Replaced special code */

}


if(ycf_gen_control_tmp_N98)
{
    {
                cret_N80_N107 = DB_ERROR_NONE_FALSE;
            }

}
else
{
{


ycf_sub_fun_trap_state_wb_N97 = NULL;
{
  int  ycf_tmp_fun_call_tmp_var;
ycf_tmp_fun_call_tmp_var =  ets_insert_2_list_from_p_heap_ycf_gen_yielding(&ycf_nr_of_reductions,
                         &ycf_sub_fun_trap_state_wb_N97,ycf_extra_context,
                         ycf_yield_alloc,ycf_yield_free,
                         ycf_yield_alloc_free_context,
                         YCF_ALLOC_NEXT_MAX_SIZE(),
                         YCF_ALLOC_NEXT_BLOCK()
                         ,tb_N90_N120, list_N91_N121);
while(YCF_IS_YIELDED(ycf_sub_fun_trap_state_wb_N97)){
   ycf_trap_location = 5;
   goto ycf_do_yield_label_ets_insert_2_list;
  ycf_yield_location_label_5:;
   ycf_tmp_fun_call_tmp_var =  ets_insert_2_list_from_p_heap_ycf_gen_continue(&ycf_nr_of_reductions,
                            &ycf_sub_fun_trap_state_wb_N97,
                            ycf_extra_context);
}

                cret_N80_N107 = ycf_tmp_fun_call_tmp_var;


}
/* YCF Replaced special code */

}


}

}

} else
{
{


ycf_sub_fun_trap_state_wb_N99 = NULL;
{
  int  ycf_tmp_fun_call_tmp_var;
ycf_tmp_fun_call_tmp_var =  ets_insert_2_list_from_p_heap_ycf_gen_yielding(&ycf_nr_of_reductions,
                         &ycf_sub_fun_trap_state_wb_N99,ycf_extra_context,
                         ycf_yield_alloc,ycf_yield_free,
                         ycf_yield_alloc_free_context,
                         YCF_ALLOC_NEXT_MAX_SIZE(),
                         YCF_ALLOC_NEXT_BLOCK()
                         ,tb_N90_N120, list_N91_N121);
while(YCF_IS_YIELDED(ycf_sub_fun_trap_state_wb_N99)){
   ycf_trap_location = 6;
   goto ycf_do_yield_label_ets_insert_2_list;
  ycf_yield_location_label_6:;
   ycf_tmp_fun_call_tmp_var =  ets_insert_2_list_from_p_heap_ycf_gen_continue(&ycf_nr_of_reductions,
                            &ycf_sub_fun_trap_state_wb_N99,
                            ycf_extra_context);
}

            cret_N80_N107 = ycf_tmp_fun_call_tmp_var;


}
/* YCF Replaced special code */

}


}
        db_unlock(tb_N90_N120, LCK_WRITE);
        YCF_SET_NR_OF_REDS_LEFT(YCF_NR_OF_REDS_LEFT() - reds_boost_N78_N116);
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

        return ets_cret_to_return_value(p_N88_N118, cret_N80_N107);
}
    }

}
{

ycf_sub_fun_trap_state_wb_N100 = NULL;
{
   void  *  ycf_tmp_fun_call_tmp_var;
ycf_tmp_fun_call_tmp_var =  ets_insert_2_list_copy_term_list_ycf_gen_yielding(&ycf_nr_of_reductions,
                         &ycf_sub_fun_trap_state_wb_N100,ycf_extra_context,
                         ycf_yield_alloc,ycf_yield_free,
                         ycf_yield_alloc_free_context,
                         YCF_ALLOC_NEXT_MAX_SIZE(),
                         YCF_ALLOC_NEXT_BLOCK()
                         ,meth_N82_N109, compressed_N83_N110, keypos_N84_N111, list_N91_N121);
while(YCF_IS_YIELDED(ycf_sub_fun_trap_state_wb_N100)){
   ycf_trap_location = 7;
   goto ycf_do_yield_label_ets_insert_2_list;
  ycf_yield_location_label_7:;
   ycf_tmp_fun_call_tmp_var =  ets_insert_2_list_copy_term_list_ycf_gen_continue(&ycf_nr_of_reductions,
                            &ycf_sub_fun_trap_state_wb_N100,
                            ycf_extra_context);
}

    /* Copy term list from heap so that other processes can help */
    db_term_list_N81_N108 = ycf_tmp_fun_call_tmp_var;


}
/* YCF Replaced special code */

}

{

ycf_sub_fun_trap_state_wb_N101 = NULL;
{
 ets_insert_2_list_lock_tbl_ycf_gen_yielding(&ycf_nr_of_reductions,
                         &ycf_sub_fun_trap_state_wb_N101,ycf_extra_context,
                         ycf_yield_alloc,ycf_yield_free,
                         ycf_yield_alloc_free_context,
                         YCF_ALLOC_NEXT_MAX_SIZE(),
                         YCF_ALLOC_NEXT_BLOCK()
                         ,table_id_N89_N119, p_N88_N118, bif_ix_N86_N113, ETS_INSERT_2_LIST_GLOBAL);
while(YCF_IS_YIELDED(ycf_sub_fun_trap_state_wb_N101)){
   ycf_trap_location = 8;
   goto ycf_do_yield_label_ets_insert_2_list;
  ycf_yield_location_label_8:;
    ets_insert_2_list_lock_tbl_ycf_gen_continue(&ycf_nr_of_reductions,
                            &ycf_sub_fun_trap_state_wb_N101,
                            ycf_extra_context);
}


}
/* YCF Replaced special code */

}

    /* The operation must complete after this point */
    if (is_insert_new_N92_N122)
{
{
{


ycf_sub_fun_trap_state_wb_N102 = NULL;
{
  int  ycf_tmp_fun_call_tmp_var;
ycf_tmp_fun_call_tmp_var =     !  ! ets_insert_new_2_dbterm_list_has_member_ycf_gen_yielding(&ycf_nr_of_reductions,
                         &ycf_sub_fun_trap_state_wb_N102,ycf_extra_context,
                         ycf_yield_alloc,ycf_yield_free,
                         ycf_yield_alloc_free_context,
                         YCF_ALLOC_NEXT_MAX_SIZE(),
                         YCF_ALLOC_NEXT_BLOCK()
                         ,tb_N90_N120, db_term_list_N81_N108);
while(YCF_IS_YIELDED(ycf_sub_fun_trap_state_wb_N102)){
   ycf_trap_location = 9;
   goto ycf_do_yield_label_ets_insert_2_list;
  ycf_yield_location_label_9:;
   ycf_tmp_fun_call_tmp_var =     !  ! ets_insert_new_2_dbterm_list_has_member_ycf_gen_continue(&ycf_nr_of_reductions,
                            &ycf_sub_fun_trap_state_wb_N102,
                            ycf_extra_context);
}

ycf_gen_control_tmp_N105 = ycf_tmp_fun_call_tmp_var;


}
/* YCF Replaced special code */

}


if(ycf_gen_control_tmp_N105)
{
    {
{


ycf_sub_fun_trap_state_wb_N103 = NULL;
{
 ets_insert_2_list_destroy_copied_dbterms_ycf_gen_yielding(&ycf_nr_of_reductions,
                         &ycf_sub_fun_trap_state_wb_N103,ycf_extra_context,
                         ycf_yield_alloc,ycf_yield_free,
                         ycf_yield_alloc_free_context,
                         YCF_ALLOC_NEXT_MAX_SIZE(),
                         YCF_ALLOC_NEXT_BLOCK()
                         ,meth_N82_N109,
                                                     compressed_N83_N110,
                                                     db_term_list_N81_N108);
while(YCF_IS_YIELDED(ycf_sub_fun_trap_state_wb_N103)){
   ycf_trap_location = 10;
   goto ycf_do_yield_label_ets_insert_2_list;
  ycf_yield_location_label_10:;
    ets_insert_2_list_destroy_copied_dbterms_ycf_gen_continue(&ycf_nr_of_reductions,
                            &ycf_sub_fun_trap_state_wb_N103,
                            ycf_extra_context);
}


}
/* YCF Replaced special code */

}


            cret_N80_N107 = DB_ERROR_NONE_FALSE;
        }

}
else
{
{


ycf_sub_fun_trap_state_wb_N104 = NULL;
{
 ets_insert_2_list_insert_db_term_list_ycf_gen_yielding(&ycf_nr_of_reductions,
                         &ycf_sub_fun_trap_state_wb_N104,ycf_extra_context,
                         ycf_yield_alloc,ycf_yield_free,
                         ycf_yield_alloc_free_context,
                         YCF_ALLOC_NEXT_MAX_SIZE(),
                         YCF_ALLOC_NEXT_BLOCK()
                         ,tb_N90_N120, db_term_list_N81_N108);
while(YCF_IS_YIELDED(ycf_sub_fun_trap_state_wb_N104)){
   ycf_trap_location = 11;
   goto ycf_do_yield_label_ets_insert_2_list;
  ycf_yield_location_label_11:;
    ets_insert_2_list_insert_db_term_list_ycf_gen_continue(&ycf_nr_of_reductions,
                            &ycf_sub_fun_trap_state_wb_N104,
                            ycf_extra_context);
}


}
/* YCF Replaced special code */

}


}

}

} else
{
{


ycf_sub_fun_trap_state_wb_N106 = NULL;
{
 ets_insert_2_list_insert_db_term_list_ycf_gen_yielding(&ycf_nr_of_reductions,
                         &ycf_sub_fun_trap_state_wb_N106,ycf_extra_context,
                         ycf_yield_alloc,ycf_yield_free,
                         ycf_yield_alloc_free_context,
                         YCF_ALLOC_NEXT_MAX_SIZE(),
                         YCF_ALLOC_NEXT_BLOCK()
                         ,tb_N90_N120, db_term_list_N81_N108);
while(YCF_IS_YIELDED(ycf_sub_fun_trap_state_wb_N106)){
   ycf_trap_location = 12;
   goto ycf_do_yield_label_ets_insert_2_list;
  ycf_yield_location_label_12:;
    ets_insert_2_list_insert_db_term_list_ycf_gen_continue(&ycf_nr_of_reductions,
                            &ycf_sub_fun_trap_state_wb_N106,
                            ycf_extra_context);
}


}
/* YCF Replaced special code */

}


}
    if (tb_N90_N120->common.continuation != NULL)
{
    {
        /* Uninstall the continuation from the table struct */
        tb_N90_N120->common.continuation = NULL;
        if (is_insert_new_N92_N122)
{
    {
            result_ptr_N79_N117 =
                ERTS_MAGIC_BIN_DATA(tb_N90_N120->common.continuation_res_bin);
            *result_ptr_N79_N117 = cret_N80_N107;
            erts_bin_release(tb_N90_N120->common.continuation_res_bin);
        }

}
        tb_N90_N120->common.status |= tb_N90_N120->common.type & (DB_PRIVATE|DB_PROTECTED|DB_PUBLIC);
        tb_N90_N120->common.status &= ~DB_BUSY;
        erts_atomic_set_relb(&tb_N90_N120->common.continuation_state, (Sint)NULL);
    }

}
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

    
    return ets_cret_to_return_value(NULL, cret_N80_N107);
}
/* YCF Replaced special code */


{
  struct gen_ycf_trap_state_for_ets_insert_2_list* ycf_my_trap_state;
  ycf_do_yield_label_ets_insert_2_list:;
  
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */
  if (*ycf_trap_state == NULL) {
    ycf_my_trap_state = ycf_yield_alloc(sizeof(struct gen_ycf_trap_state_for_ets_insert_2_list), ycf_yield_alloc_free_context);
      } else {
    ycf_my_trap_state = *ycf_trap_state;
  }
      ycf_my_trap_state->ycf_yield_alloc = ycf_yield_alloc;
    ycf_my_trap_state->ycf_yield_free = ycf_yield_free;
    ycf_my_trap_state->ycf_yield_alloc_free_context = ycf_yield_alloc_free_context;
    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size = ycf_stack_alloc_size_or_max_size;
    ycf_my_trap_state->ycf_stack_alloc_data = ycf_stack_alloc_data;
    ycf_my_trap_state->ycf_trap_location = ycf_trap_location;
    ycf_my_trap_state->ycf_nr_of_reductions = ycf_nr_of_reductions;
    ycf_my_trap_state->ycf_frame_alloc_data = ycf_frame_alloc_data;
    ycf_my_trap_state->cret_N80_N107 = cret_N80_N107;
    ycf_my_trap_state->db_term_list_N81_N108 = db_term_list_N81_N108;
    ycf_my_trap_state->meth_N82_N109 = meth_N82_N109;
    ycf_my_trap_state->compressed_N83_N110 = compressed_N83_N110;
    ycf_my_trap_state->keypos_N84_N111 = keypos_N84_N111;
    ycf_my_trap_state->tb_type_N85_N112 = tb_type_N85_N112;
    ycf_my_trap_state->bif_ix_N86_N113 = bif_ix_N86_N113;
    ycf_my_trap_state->list_len_N87_N114 = list_len_N87_N114;
    ycf_my_trap_state->ret_N77_N115 = ret_N77_N115;
    ycf_my_trap_state->reds_boost_N78_N116 = reds_boost_N78_N116;
    ycf_my_trap_state->result_ptr_N79_N117 = result_ptr_N79_N117;
    ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N93 = ycf_sub_fun_trap_state_wb_N93;
    ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N94 = ycf_sub_fun_trap_state_wb_N94;
    ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N95 = ycf_sub_fun_trap_state_wb_N95;
    ycf_my_trap_state->ycf_gen_control_tmp_N98 = ycf_gen_control_tmp_N98;
    ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N96 = ycf_sub_fun_trap_state_wb_N96;
    ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N97 = ycf_sub_fun_trap_state_wb_N97;
    ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N99 = ycf_sub_fun_trap_state_wb_N99;
    ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N100 = ycf_sub_fun_trap_state_wb_N100;
    ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N101 = ycf_sub_fun_trap_state_wb_N101;
    ycf_my_trap_state->ycf_gen_control_tmp_N105 = ycf_gen_control_tmp_N105;
    ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N102 = ycf_sub_fun_trap_state_wb_N102;
    ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N103 = ycf_sub_fun_trap_state_wb_N103;
    ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N104 = ycf_sub_fun_trap_state_wb_N104;
    ycf_my_trap_state->ycf_sub_fun_trap_state_wb_N106 = ycf_sub_fun_trap_state_wb_N106;
    ycf_my_trap_state->p_N88_N118 = p_N88_N118;
    ycf_my_trap_state->table_id_N89_N119 = table_id_N89_N119;
    ycf_my_trap_state->tb_N90_N120 = tb_N90_N120;
    ycf_my_trap_state->list_N91_N121 = list_N91_N121;
    ycf_my_trap_state->is_insert_new_N92_N122 = is_insert_new_N92_N122;

  *ycf_nr_of_reductions_param = ycf_nr_of_reductions;
  *ycf_trap_state = ycf_my_trap_state;
      {
    static   BIF_RETTYPE  const ycf_unused_ret_value;
    return ycf_unused_ret_value;
  }

}

}
/* clang-format off */
YCF_GCC_DIAG_ON(uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */

#undef YCF_STACK_ALLOC
#undef YCF_IN_YIELDING_FUN



struct gen_ycf_trap_state_for_ets_insert_2_list_lock_tbl {
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    BIF_RETTYPE fail_ret_N125_N131;
    DbTable* tb_N126_N132;
            ets_insert_2_list_info *ctx_N123_N133;
            ets_insert_2_list_info *ctx_N124_N134;Eterm table_id_N127_N135;
                                       Process* p_N128_N136;
                                       Uint bif_ix_N129_N137;
                                       ets_insert_2_list_status on_success_status_N130_N138;
};


/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

static void ets_insert_2_list_lock_tbl_ycf_gen_destroy(void* ycf_my_trap_state_param){
     {
     struct gen_ycf_trap_state_for_ets_insert_2_list_lock_tbl* ycf_my_trap_state = ycf_my_trap_state_param;
     
ycf_yield_alloc_type ycf_yield_alloc;
ycf_yield_free_type ycf_yield_free;
void* ycf_yield_alloc_free_context;
size_t ycf_stack_alloc_size_or_max_size;
void* ycf_stack_alloc_data;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;
    BIF_RETTYPE fail_ret_N125_N131;
    DbTable* tb_N126_N132;
            ets_insert_2_list_info *ctx_N123_N133;
            ets_insert_2_list_info *ctx_N124_N134;Eterm table_id_N127_N135;
                                       Process* p_N128_N136;
                                       Uint bif_ix_N129_N137;
                                       ets_insert_2_list_status on_success_status_N130_N138;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    fail_ret_N125_N131 = ycf_my_trap_state->fail_ret_N125_N131;
    tb_N126_N132 = ycf_my_trap_state->tb_N126_N132;
    ctx_N123_N133 = ycf_my_trap_state->ctx_N123_N133;
    ctx_N124_N134 = ycf_my_trap_state->ctx_N124_N134;
    table_id_N127_N135 = ycf_my_trap_state->table_id_N127_N135;
    p_N128_N136 = ycf_my_trap_state->p_N128_N136;
    bif_ix_N129_N137 = ycf_my_trap_state->bif_ix_N129_N137;
    on_success_status_N130_N138 = ycf_my_trap_state->on_success_status_N130_N138;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_destroy_stack_allocator(&ycf_frame_alloc_data, ycf_yield_free, ycf_yield_alloc_free_context);
     ycf_yield_free(ycf_my_trap_state, ycf_yield_alloc_free_context);
     }
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unused-but-set-variable)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */


/* clang-format off */
YCF_GCC_DIAG_OFF(unused-function)
/* clang-format on */

   static  void  ets_insert_2_list_lock_tbl_ycf_gen_continue(long* ycf_number_of_reduction_param,
                       void** ycf_trap_state,
                       void* ycf_extra_context){
     struct gen_ycf_trap_state_for_ets_insert_2_list_lock_tbl* ycf_my_trap_state = *ycf_trap_state;
ets_insert_2_list_lock_tbl_ycf_gen_yielding(ycf_number_of_reduction_param,
                    ycf_trap_state,
                    ycf_extra_context,
                    ycf_my_trap_state->ycf_yield_alloc,
                    ycf_my_trap_state->ycf_yield_free,
                    ycf_my_trap_state->ycf_yield_alloc_free_context,
                    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size,
                    ycf_my_trap_state->ycf_stack_alloc_data
                    ,ycf_my_trap_state->table_id_N127_N135,ycf_my_trap_state->p_N128_N136,ycf_my_trap_state->bif_ix_N129_N137,ycf_my_trap_state->on_success_status_N130_N138)
;
return;
}
/* clang-format off */
YCF_GCC_DIAG_ON(unused-function)
/* clang-format on */

#define YCF_IN_YIELDING_FUN 1
#undef YCF_STACK_ALLOC
#define YCF_STACK_ALLOC(SIZE) \
    ycf_stack_alloc(SIZE,\
                    &ycf_frame_alloc_data,\
                    ycf_yield_alloc, ycf_yield_alloc_free_context,\
                     ycf_stack_alloc_size_or_max_size)

/* clang-format off */
YCF_GCC_DIAG_OFF(pragmas)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_OFF(uninitialized)
/* clang-format on */


static void ets_insert_2_list_lock_tbl_ycf_gen_yielding(
long* ycf_nr_of_reductions_param,
void** ycf_trap_state,
void* ycf_extra_context,
ycf_yield_alloc_type ycf_yield_alloc,
ycf_yield_free_type ycf_yield_free,
void* ycf_yield_alloc_free_context,
size_t ycf_stack_alloc_size_or_max_size,
void* ycf_stack_alloc_data,Eterm table_id_N127_N135,
                                       Process* p_N128_N136,
                                       Uint bif_ix_N129_N137,
                                       ets_insert_2_list_status on_success_status_N130_N138)
{
    BIF_RETTYPE fail_ret_N125_N131;
    DbTable* tb_N126_N132;
            ets_insert_2_list_info *ctx_N123_N133;
            ets_insert_2_list_info *ctx_N124_N134;
int ycf_trap_location;
long ycf_nr_of_reductions;
struct ycf_alloc_data ycf_frame_alloc_data;

{
  
  ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
  ycf_frame_alloc_data.size = 0;
  ycf_frame_alloc_data.max_size = ycf_stack_alloc_size_or_max_size;
  ycf_frame_alloc_data.data = ycf_stack_alloc_data;
  ycf_frame_alloc_data.needs_freeing = 0;
  if(*ycf_trap_state != NULL){
     struct gen_ycf_trap_state_for_ets_insert_2_list_lock_tbl* ycf_my_trap_state = *ycf_trap_state;
     
    ycf_yield_alloc = ycf_my_trap_state->ycf_yield_alloc;
    ycf_yield_free = ycf_my_trap_state->ycf_yield_free;
    ycf_yield_alloc_free_context = ycf_my_trap_state->ycf_yield_alloc_free_context;
    ycf_stack_alloc_size_or_max_size = ycf_my_trap_state->ycf_stack_alloc_size_or_max_size;
    ycf_stack_alloc_data = ycf_my_trap_state->ycf_stack_alloc_data;
    ycf_trap_location = ycf_my_trap_state->ycf_trap_location;
    ycf_nr_of_reductions = ycf_my_trap_state->ycf_nr_of_reductions;
    ycf_frame_alloc_data = ycf_my_trap_state->ycf_frame_alloc_data;
    fail_ret_N125_N131 = ycf_my_trap_state->fail_ret_N125_N131;
    tb_N126_N132 = ycf_my_trap_state->tb_N126_N132;
    ctx_N123_N133 = ycf_my_trap_state->ctx_N123_N133;
    ctx_N124_N134 = ycf_my_trap_state->ctx_N124_N134;
    table_id_N127_N135 = ycf_my_trap_state->table_id_N127_N135;
    p_N128_N136 = ycf_my_trap_state->p_N128_N136;
    bif_ix_N129_N137 = ycf_my_trap_state->bif_ix_N129_N137;
    on_success_status_N130_N138 = ycf_my_trap_state->on_success_status_N130_N138;

     
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

     ycf_nr_of_reductions = *ycf_nr_of_reductions_param;
     switch(ycf_trap_location){
case 1: goto ycf_yield_location_label_1;

}
  }
}
{

}
    do
{
    {
        fail_ret_N125_N131 = db_get_table_or_fail_return(&tb_N126_N132,
                                               table_id_N127_N135,
                                               DB_WRITE,
                                               LCK_WRITE,
                                               bif_ix_N129_N137,
                                               p_N128_N136);
        if (tb_N126_N132 == NULL)
{
    {
            ctx_N123_N133 = YCF_GET_EXTRA_CONTEXT();
            if (p_N128_N136->freason == TRAP)
{
    {
                ctx_N123_N133->status = ETS_INSERT_2_LIST_FAILED_TO_GET_LOCK;
            }

} else
{
    {
                ctx_N123_N133->status = ETS_INSERT_2_LIST_FAILED_TO_GET_LOCK_DESTROY;
                ctx_N123_N133->destroy_return_value = fail_ret_N125_N131;
            }

}
#ifdef DEBUG
            /*
             *  Setting ctx to NULL to avoid that YCF crashes with a
             *  pointer to stack error when running a debug
             *  build. YCF_GET_EXTRA_CONTEXT() may change between
             *  yields as we use stack allocated data for the context
             *  before the first yield so it is important that the
             *  context is obtained again with YCF_GET_EXTRA_CONTEXT()
             *  if a yield might have happened.
             */
            ctx_N123_N133 = NULL;
#endif


{
  ycf_nr_of_reductions = 0;
  ycf_trap_location = 1;
  goto ycf_do_yield_label_ets_insert_2_list_lock_tbl;
  ycf_yield_location_label_1:;
}
        }

} else
{
    {
            ctx_N124_N134 = YCF_GET_EXTRA_CONTEXT();
            ctx_N124_N134->status = on_success_status_N130_N138;
            ASSERT(DB_LOCK_FREE(tb_N126_N132) || erts_lc_rwmtx_is_rwlocked(&tb_N126_N132->common.rwlock));
            ASSERT(!(tb_N126_N132->common.status & DB_DELETE));
        }

}
    }

} while (tb_N126_N132 == NULL);
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

return;
}
{

if (*ycf_trap_state != NULL) {
   ycf_yield_free(*ycf_trap_state, ycf_yield_alloc_free_context);
   *ycf_trap_state = NULL;
}
ycf_destroy_stack_allocator(&ycf_frame_alloc_data,
                            ycf_yield_free,
                            ycf_yield_alloc_free_context);
*ycf_nr_of_reductions_param = ycf_nr_of_reductions;
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */

return;
}

{
  struct gen_ycf_trap_state_for_ets_insert_2_list_lock_tbl* ycf_my_trap_state;
  ycf_do_yield_label_ets_insert_2_list_lock_tbl:;
  
/* YCF SPECIAL CUSTOM CODE START */

/* YCF SPECIAL CUSTOM CODE END */
  if (*ycf_trap_state == NULL) {
    ycf_my_trap_state = ycf_yield_alloc(sizeof(struct gen_ycf_trap_state_for_ets_insert_2_list_lock_tbl), ycf_yield_alloc_free_context);
      } else {
    ycf_my_trap_state = *ycf_trap_state;
  }
      ycf_my_trap_state->ycf_yield_alloc = ycf_yield_alloc;
    ycf_my_trap_state->ycf_yield_free = ycf_yield_free;
    ycf_my_trap_state->ycf_yield_alloc_free_context = ycf_yield_alloc_free_context;
    ycf_my_trap_state->ycf_stack_alloc_size_or_max_size = ycf_stack_alloc_size_or_max_size;
    ycf_my_trap_state->ycf_stack_alloc_data = ycf_stack_alloc_data;
    ycf_my_trap_state->ycf_trap_location = ycf_trap_location;
    ycf_my_trap_state->ycf_nr_of_reductions = ycf_nr_of_reductions;
    ycf_my_trap_state->ycf_frame_alloc_data = ycf_frame_alloc_data;
    ycf_my_trap_state->fail_ret_N125_N131 = fail_ret_N125_N131;
    ycf_my_trap_state->tb_N126_N132 = tb_N126_N132;
    ycf_my_trap_state->ctx_N123_N133 = ctx_N123_N133;
    ycf_my_trap_state->ctx_N124_N134 = ctx_N124_N134;
    ycf_my_trap_state->table_id_N127_N135 = table_id_N127_N135;
    ycf_my_trap_state->p_N128_N136 = p_N128_N136;
    ycf_my_trap_state->bif_ix_N129_N137 = bif_ix_N129_N137;
    ycf_my_trap_state->on_success_status_N130_N138 = on_success_status_N130_N138;

  *ycf_nr_of_reductions_param = ycf_nr_of_reductions;
  *ycf_trap_state = ycf_my_trap_state;
    return;

}
}
/* clang-format off */
YCF_GCC_DIAG_ON(uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(maybe-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(sometimes-uninitialized)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(unknown-warning-option)
/* clang-format on */

/* clang-format off */
YCF_GCC_DIAG_ON(pragmas)
/* clang-format on */

#undef YCF_STACK_ALLOC
#undef YCF_IN_YIELDING_FUN
